@@ -31,7 +31,6 @@
  *	$Id$
  */
 
-#include  <X11/keysym.h>	/* XK_xxx */
 #include  <kiklib/kik_mem.h>	/* malloc/alloca/free */
 #include  <kiklib/kik_str.h>	/* kik_str_alloca_dup kik_snprintf kik_str_sep*/
 #include  <kiklib/kik_locale.h>	/* kik_get_lang */
@@ -130,7 +129,7 @@ xksym_to_msymbol(
 	int  is_super ;
 	int  is_hyper ;
 
-	if( XK_Shift_L <= ksym && ksym <= XK_Hyper_R)
+	if( IsModifierKey(ksym))
 	{
 		return  Mnil ;
 	}
@@ -143,7 +142,7 @@ xksym_to_msymbol(
 						&is_hyper) ;
 
 	/* Latin 1 */
-	if( XK_space <= ksym && ksym <= XK_asciitilde)
+	if( 0x20 <= ksym && ksym <= 0x7e)
 	{
 		char  buf[2] = " ";
 		buf[0] = ksym ;
@@ -157,6 +156,9 @@ xksym_to_msymbol(
 		return  msymbol( buf) ;
 	}
 
+#ifdef  USE_FRAMEBUFFER
+	return  Mnil ;
+#else
 	if( is_shift)
 		filled_len += kik_snprintf( &mod[filled_len] ,
 				    sizeof(mod) - filled_len ,
@@ -201,6 +203,7 @@ xksym_to_msymbol(
 	kik_snprintf( str , len , "%s%s" , mod , key) ;
 
 	return  msymbol( str) ;
+#endif
 }
 
 static MInputMethod *
@@ -485,17 +488,21 @@ preedit_changed(
 		int  is_biwidth = 0 ;
 		int  is_comb = 0 ;
 
-		if( ch.cs == ISO10646_UCS4_1)
+		if( (*syms->ml_convert_to_internal_ch)( &ch ,
+			(*m17nlib->im.listener->get_unicode_policy)(m17nlib->im.listener->self) ,
+			US_ASCII) <= 0)
 		{
-			if( ch.property & MKF_BIWIDTH)
-			{
-				is_biwidth = 1 ;
-			}
-			else if( ch.property & MKF_AWIDTH)
-			{
-				/* TODO: check col_size_of_width_a */
-				is_biwidth = 1 ;
-			}
+			continue ;
+		}
+
+		if( ch.property & MKF_BIWIDTH)
+		{
+			is_biwidth = 1 ;
+		}
+		else if( ch.property & MKF_AWIDTH)
+		{
+			/* TODO: check col_size_of_width_a */
+			is_biwidth = 1 ;
 		}
 
 		if( m17nlib->input_context->candidate_list &&
@@ -510,10 +517,7 @@ preedit_changed(
 		if( ch.property & MKF_COMBINING)
 		{
 			is_comb = 1 ;
-		}
 
-		if( is_comb)
-		{
 			if( (*syms->ml_char_combine)( p - 1 , ch.ch ,
 						      ch.size , ch.cs ,
 						      is_biwidth ,
@@ -523,7 +527,7 @@ preedit_changed(
 						      0 , is_underline))
 			{
 				pos++ ;
-				continue;
+				continue ;
 			}
 
 			/*
@@ -531,11 +535,6 @@ preedit_changed(
 			 */
 		}
 
-		if( (*syms->ml_is_msb_set)( ch.cs))
-		{
-			SET_MSB( ch.ch[0]) ;
-		}
-
 		(*syms->ml_char_set)( p , ch.ch , ch.size , ch.cs ,
 				      is_biwidth , is_comb ,
 				      fg_color , bg_color ,
@@ -620,10 +619,9 @@ candidates_changed(
 		}
 
 		if( ! ( m17nlib->im.cand_screen = (*syms->x_im_candidate_screen_new)(
-				(*m17nlib->im.listener->get_display)(m17nlib->im.listener->self) ,
-				(*m17nlib->im.listener->get_font_man)(m17nlib->im.listener->self) ,
-				(*m17nlib->im.listener->get_color_man)(m17nlib->im.listener->self) ,
+				m17nlib->im.disp , m17nlib->im.font_man , m17nlib->im.color_man ,
 				(*m17nlib->im.listener->is_vertical)(m17nlib->im.listener->self) ,
+				(*m17nlib->im.listener->get_unicode_policy)(m17nlib->im.listener->self) ,
 				is_vertical_direction ,
 				(*m17nlib->im.listener->get_line_height)(m17nlib->im.listener->self) ,
 				x , y)))
@@ -836,10 +834,10 @@ switch_mode(
 		if( m17nlib->im.stat_screen == NULL)
 		{
 			if( ! ( m17nlib->im.stat_screen = (*syms->x_im_status_screen_new)(
-					(*m17nlib->im.listener->get_display)(m17nlib->im.listener->self) ,
-					(*m17nlib->im.listener->get_font_man)(m17nlib->im.listener->self) ,
-					(*m17nlib->im.listener->get_color_man)(m17nlib->im.listener->self) ,
+					m17nlib->im.disp , m17nlib->im.font_man ,
+					m17nlib->im.color_man ,
 					(*m17nlib->im.listener->is_vertical)(m17nlib->im.listener->self) ,
+					(*m17nlib->im.listener->get_line_height)(m17nlib->im.listener->self) ,
 					x , y)))
 			{
 			#ifdef  DEBUG
