@@ -36,13 +36,13 @@
 #include  <uim-helper.h>
 #include  <uim-im-switcher.h>
 
-#include  <X11/keysym.h>	/* XK_xxx */
 #include  <kiklib/kik_mem.h>	/* malloc/alloca/free */
 #include  <kiklib/kik_str.h>	/* kik_str_alloca_dup kik_str_sep kik_snprintf*/
 #include  <kiklib/kik_locale.h>	/* kik_get_locale */
 #include  <kiklib/kik_list.h>
 
 #include  <x_im.h>
+
 #include  "../im_common.h"
 #include  "../im_info.h"
 
@@ -149,7 +149,7 @@ xksym_to_ukey(
 	)
 {
 	/* Latin 1 */
-	if( XK_space <= ksym && ksym <= XK_asciitilde)
+	if( 0x20 <= ksym && ksym <= 0x7e)
 	{
 		return  ksym ;
 	}
@@ -167,9 +167,11 @@ xksym_to_ukey(
 		return  UKey_Escape ;
 	case  XK_Delete:
 		return  UKey_Delete ;
+#ifdef  XK_Multi_key
 	/* International & multi-key character composition */
 	case  XK_Multi_key:
 		return  UKey_Multi_key ;
+#endif
 	/* Japanese keyboard support */
 	case  XK_Muhenkan:
 		return  UKey_Muhenkan ;
@@ -177,6 +179,8 @@ xksym_to_ukey(
 		return  UKey_Henkan_Mode ;
 	case  XK_Zenkaku_Hankaku:
 		return  UKey_Zenkaku_Hankaku ;
+	case  XK_Hiragana_Katakana:
+		return  UKey_Hiragana_Katakana ;
 	/* Cursor control & motion */
 	case  XK_Home:
 		return  UKey_Home ;
@@ -242,6 +246,7 @@ xksym_to_ukey(
 		return  UKey_F23 ;
 	case  XK_F24:
 		return  UKey_F24 ;
+#ifdef  XK_F25
 	case  XK_F25:
 		return  UKey_F25 ;
 	case  XK_F26:
@@ -264,12 +269,19 @@ xksym_to_ukey(
 		return  UKey_F34 ;
 	case  XK_F35:
 		return  UKey_F35 ;
+#endif
+#ifdef  XK_KP_Space
 	case  XK_KP_Space:
 		return  ' ' ;
+#endif
+#ifdef  XK_KP_Tab
 	case  XK_KP_Tab:
 		return  UKey_Tab ;
+#endif
+#ifdef  XK_KP_Enter
 	case  XK_KP_Enter:
 		return  UKey_Return ;
+#endif
 	case  XK_KP_F1:
 		return  UKey_F1 ;
 	case  XK_KP_F2:
@@ -296,8 +308,10 @@ xksym_to_ukey(
 		return  UKey_End ;
 	case  XK_KP_Delete:
 		return  UKey_Delete ;
+#ifdef  XK_KP_Equal
 	case  XK_KP_Equal:
 		return  '=' ;
+#endif
 	case  XK_KP_Multiply:
 		return  '*' ;
 	case  XK_KP_Add:
@@ -627,33 +641,34 @@ preedit_pushback(
 		int  is_biwidth = 0 ;
 		int  is_comb = 0 ;
 
-		if( ch.cs == ISO10646_UCS4_1)
+		if( (*syms->ml_convert_to_internal_ch)( &ch ,
+			(*uim->im.listener->get_unicode_policy)(uim->im.listener->self) ,
+			US_ASCII) <= 0)
 		{
-			if( ch.property & MKF_BIWIDTH)
-			{
-				is_biwidth = 1 ;
-			}
-			else if( ch.property & MKF_AWIDTH)
-			{
-				/* TODO: check col_size_of_width_a */
-				is_biwidth = 1 ;
-			}
+			continue ;
+		}
+
+		if( ch.property & MKF_BIWIDTH)
+		{
+			is_biwidth = 1 ;
+		}
+		else if( ch.property & MKF_AWIDTH)
+		{
+			/* TODO: check col_size_of_width_a */
+			is_biwidth = 1 ;
 		}
 
 		if( ch.property & MKF_COMBINING)
 		{
 			is_comb = 1 ;
-		}
 
-		if( is_comb)
-		{
 			if( (*syms->ml_char_combine)( p - 1 , ch.ch ,
 						      ch.size , ch.cs ,
 						      is_biwidth , is_comb ,
 						      fg_color , bg_color ,
 						      0 , is_underline))
 			{
-				continue;
+				continue ;
 			}
 
 			/*
@@ -661,11 +676,6 @@ preedit_pushback(
 			 */
 		}
 
-		if( (*syms->ml_is_msb_set)( ch.cs))
-		{
-			SET_MSB( ch.ch[0]) ;
-		}
-
 		(*syms->ml_char_set)( p , ch.ch , ch.size , ch.cs ,
 				      is_biwidth , is_comb ,
 				      fg_color , bg_color ,
@@ -743,11 +753,10 @@ candidate_activate(
 					       &x , &y) ;
 
 		if( ! ( uim->im.cand_screen = (*syms->x_im_candidate_screen_new)(
-				(*uim->im.listener->get_display)(uim->im.listener->self) ,
-				(*uim->im.listener->get_font_man)(uim->im.listener->self) ,
-				(*uim->im.listener->get_color_man)(uim->im.listener->self) ,
+				uim->im.disp , uim->im.font_man , uim->im.color_man ,
 				(*uim->im.listener->is_vertical)(uim->im.listener->self) ,
 				1 ,
+				(*uim->im.listener->get_unicode_policy)(uim->im.listener->self) ,
 				(*uim->im.listener->get_line_height)(uim->im.listener->self) ,
 				x , y)))
 		{
@@ -777,15 +786,19 @@ candidate_activate(
 		uim_candidate  c ;
 		u_char *  _p ;
 		u_char *  p = NULL ;
-		const char * heading;
-		u_int info;
+		const char *  heading ;
+		u_int info ;
 
 		c = uim_get_candidate( uim->context , i , i) ;
 		_p = (u_char*)uim_candidate_get_cand_str( c) ;
-		heading = uim_candidate_get_heading_label( c);
-		if(heading && (heading[0])){
-			info = (((u_int)heading[0]) << 16) | i;
-		}else{
+		heading = uim_candidate_get_heading_label( c) ;
+		if( heading && heading[0])
+		{
+			/* heading[1] may be '\0' */
+			info = (((u_int)heading[0]) << 16) | (((u_int)heading[1]) << 24) | i;
+		}
+		else
+		{
 			info = (((u_int)' ') << 16) | i;
 		}
 
@@ -956,8 +969,8 @@ delete(
 
 	if( ref_count == 0 && initialized)
 	{
-		uim_helper_close_client_fd( helper_fd) ;
 		(*syms->x_event_source_remove_fd)( helper_fd) ;
+		uim_helper_close_client_fd( helper_fd) ;
 		helper_fd = -1 ;
 
 		uim_quit() ;
@@ -1009,6 +1022,9 @@ key_event(
 		kik_msg_printf( ">>ksym            : %.8x\n" , ksym) ;
 	}
 
+#ifdef  USE_FRAMEBUFFER
+	uim->pressing_mod_key = ~0 ;
+#else
 	if( ! ( event->state & uim->mod_ignore_mask))
 	{
 		uim->pressing_mod_key = 0 ;
@@ -1043,6 +1059,7 @@ key_event(
 	default:
 		break ;
 	}
+#endif
 
 	(*uim->im.listener->compare_key_state_with_modmap)(
 							uim->im.listener->self ,
@@ -1640,10 +1657,10 @@ im_uim_new(
 error:
 	if( helper_fd != -1)
 	{
-		uim_helper_close_client_fd( helper_fd) ;
-
 		(*syms->x_event_source_remove_fd)( helper_fd) ;
 
+		uim_helper_close_client_fd( helper_fd) ;
+
 		helper_fd = -1 ;
 	}
 
