@@ -48,6 +48,10 @@
 #define  TUNEUP_HACK
 #endif
 
+#if  0
+#define  USE_LOCAL_ECHO_BY_DEFAULT
+#endif
+
 #if  0
 #define  __DEBUG
 #endif
@@ -79,6 +83,10 @@ static mkf_conv_t *  utf16_conv ;
 static char *  plink ;
 #endif
 
+#ifdef  USE_LIBSSH2
+static u_int  keepalive_interval ;
+#endif
+
 
 /* --- static functions --- */
 
@@ -236,6 +244,33 @@ set_config(
 			kik_set_msg_log_file_name( NULL) ;
 		}
 	}
+	else if( strcmp( key , "use_local_echo") == 0)
+	{
+		int  flag ;
+
+		if( ( flag = true_or_false( value)) != -1)
+		{
+			ml_term_set_use_local_echo( nativeObj->term , flag) ;
+		}
+	}
+	else if( strcmp( key , "use_alt_buffer") == 0)
+	{
+		int  flag ;
+
+		if( ( flag = true_or_false( value)) != -1)
+		{
+			ml_set_use_alt_buffer( flag) ;
+		}
+	}
+	else if( strcmp( key , "use_ansi_colors") == 0)
+	{
+		int  flag ;
+
+		if( ( flag = true_or_false( value)) != -1)
+		{
+			ml_set_use_ansi_colors( flag) ;
+		}
+	}
 }
 
 static void
@@ -355,6 +390,17 @@ get_config(
 			value = "false" ;
 		}
 	}
+	else if( strcmp( key , "use_local_echo") == 0)
+	{
+		if( ml_term_is_using_local_echo( nativeObj->term))
+		{
+			value = "true" ;
+		}
+		else
+		{
+			value = "false" ;
+		}
+	}
 
 	if( ! value)
 	{
@@ -572,6 +618,20 @@ get_num_of_filled_chars_except_spaces(
 	}
 }
 
+static void
+draw_cursor(
+	ml_term_t *  term ,
+	int (*func)( ml_line_t * , int)
+	)
+{
+	ml_line_t *  line ;
+
+	if( ( line = ml_term_get_cursor_line( term)))
+	{
+		(*func)( line , ml_term_cursor_char_index( term)) ;
+	}
+}
+
 
 /* --- global functions --- */
 
@@ -683,6 +743,7 @@ Java_mlterm_MLTermPty_nativeOpen(
 	static int  use_multi_col_char ;
 	static int  use_login_shell ;
 	static int  logging_vt_seq ;
+	static int  use_local_echo ;
 	static char *  public_key ;
 	static char *  private_key ;
 	static char **  default_argv ;
@@ -753,6 +814,11 @@ Java_mlterm_MLTermPty_nativeOpen(
 		use_char_combining = 1 ;
 		use_multi_col_char = 1 ;
 		use_login_shell = 0 ;
+	#ifdef  USE_LOCAL_ECHO_BY_DEFAULT
+		use_local_echo = 1 ;
+	#else
+		use_local_echo = 0 ;
+	#endif
 
 		if( ( conf = kik_conf_new()))
 		{
@@ -873,6 +939,22 @@ Java_mlterm_MLTermPty_nativeOpen(
 				}
 			}
 
+			if( ( value = kik_conf_get_value( conf , "use_local_echo")))
+			{
+			#ifdef  USE_LOCAL_ECHO_BY_DEFAULT
+				if( strcmp( value , "false") == 0)
+				{
+					use_local_echo = 0 ;
+				}
+			#else
+				if( strcmp( value , "true") == 0)
+				{
+					use_local_echo = 1 ;
+				}
+			#endif
+			}
+
+		#ifdef  USE_LIBSSH2
 			if( ( value = kik_conf_get_value( conf , "ssh_public_key")))
 			{
 				public_key = strdup( value) ;
@@ -888,6 +970,24 @@ Java_mlterm_MLTermPty_nativeOpen(
 				ml_pty_ssh_set_cipher_list( strdup( value)) ;
 			}
 
+			if( ( value = kik_conf_get_value( conf , "ssh_keepalive_interval")))
+			{
+				if( kik_str_to_uint( &keepalive_interval , value) &&
+				    keepalive_interval > 0)
+				{
+					ml_pty_ssh_set_keepalive_interval( keepalive_interval) ;
+				}
+			}
+
+			if( ( value = kik_conf_get_value( conf , "ssh_x11_forwarding")))
+			{
+				if( strcmp( value , "true") == 0)
+				{
+					ml_pty_ssh_set_use_x11_forwarding( 1) ;
+				}
+			}
+		#endif
+
 		#if  0
 			/* XXX How to get password ? */
 			if( ( value = kik_conf_get_value( conf , "default_server")))
@@ -952,7 +1052,7 @@ Java_mlterm_MLTermPty_nativeOpen(
 					0 /* use_bidi */ , 0 /* bidi_mode */ ,
 					0 /* use_ind */ , 1 /* use_bce */ ,
 					0 /* use_dynamic_comb */ , BSM_STATIC ,
-					0 /* vertical_mode */)))
+					0 /* vertical_mode */ , use_local_echo , NULL , NULL)))
 	{
 		goto  error ;
 	}
@@ -962,55 +1062,6 @@ Java_mlterm_MLTermPty_nativeOpen(
 		ml_term_set_logging_vt_seq( nativeObj->term , logging_vt_seq) ;
 	}
 
-	if( jarray_argv)
-	{
-		jsize  len ;
-		jsize  count ;
-
-		len = (*env)->GetArrayLength( env , jarray_argv) ;
-		argv = alloca( sizeof(char*) * (len + 1)) ;
-
-		for( count = 0 ; count < len ; count++)
-		{
-			argv[count] = (*env)->GetStringUTFChars( env ,
-						(*env)->GetObjectArrayElement( env ,
-							jarray_argv , count) ,
-						NULL) ;
-		}
-		argv[count] = NULL ;
-		cmd_path = argv[0] ;
-	}
-	else if( default_argv)
-	{
-		argv = default_argv ;
-		cmd_path = default_cmd_path ;
-	}
-	else
-	{
-	#ifdef  USE_WIN32API
-		cmd_path = NULL ;
-		argv = alloca( sizeof(char*)) ;
-		argv[0] = NULL ;
-	#else	/* USE_WIN32API */
-		cmd_path = getenv( "SHELL") ;
-		argv = alloca( sizeof(char*) * 2) ;
-		argv[1] = NULL ;
-	#endif	/* USE_WIN32API */
-	}
-
-	if( cmd_path)
-	{
-		if( use_login_shell)
-		{
-			argv[0] = alloca( strlen( cmd_path) + 2) ;
-			sprintf( argv[0] , "-%s" , kik_basename( cmd_path)) ;
-		}
-		else
-		{
-			argv[0] = kik_basename( cmd_path) ;
-		}
-	}
-
 	nativeObj->pty_listener.self = nativeObj ;
 	nativeObj->pty_listener.closed = pty_closed ;
 
@@ -1049,6 +1100,62 @@ Java_mlterm_MLTermPty_nativeOpen(
 		pass = NULL ;
 	}
 
+	if( jarray_argv)
+	{
+		jsize  len ;
+		jsize  count ;
+
+		len = (*env)->GetArrayLength( env , jarray_argv) ;
+		argv = alloca( sizeof(char*) * (len + 1)) ;
+
+		for( count = 0 ; count < len ; count++)
+		{
+			argv[count] = (*env)->GetStringUTFChars( env ,
+						(*env)->GetObjectArrayElement( env ,
+							jarray_argv , count) ,
+						NULL) ;
+		}
+		argv[count] = NULL ;
+		cmd_path = argv[0] ;
+	}
+	else if( default_argv)
+	{
+		argv = default_argv ;
+		cmd_path = default_cmd_path ;
+	}
+	else
+	{
+	#ifndef  USE_WIN32API
+		if( pass)
+	#endif
+		{
+			cmd_path = NULL ;
+			argv = alloca( sizeof(char*)) ;
+			argv[0] = NULL ;
+		}
+	#ifndef  USE_WIN32API
+		else
+		{
+			cmd_path = getenv( "SHELL") ;
+			argv = alloca( sizeof(char*) * 2) ;
+			argv[1] = NULL ;
+		}
+	#endif
+	}
+
+	if( cmd_path)
+	{
+		if( use_login_shell)
+		{
+			argv[0] = alloca( strlen( cmd_path) + 2) ;
+			sprintf( argv[0] , "-%s" , kik_basename( cmd_path)) ;
+		}
+		else
+		{
+			argv[0] = kik_basename( cmd_path) ;
+		}
+	}
+
 	envv[0] = alloca( 8 + strlen( host) + 1) ;
 	sprintf( envv[0] , "DISPLAY=%s" , host) ;
 	envv[1] = "TERM=xterm" ;
@@ -1169,6 +1276,11 @@ Java_mlterm_MLTermPty_waitForReading(
 	int  maxfd ;
 	int  ptyfd ;
 	fd_set  read_fds ;
+#ifdef  USE_LIBSSH2
+	struct timeval  tval ;
+	int *  xssh_fds ;
+	u_int  num_of_xssh_fds ;
+#endif
 
 	ml_close_dead_terms() ;
 	num_of_terms = ml_get_all_terms( &terms) ;
@@ -1178,21 +1290,63 @@ Java_mlterm_MLTermPty_waitForReading(
 		return  JNI_FALSE ;
 	}
 
-	maxfd = 0 ;
-	FD_ZERO( &read_fds) ;
+#ifdef  USE_LIBSSH2
+	num_of_xssh_fds = ml_pty_ssh_get_x11_fds( &xssh_fds) ;
+#endif
 
-	for( count = 0 ; count < num_of_terms ; count ++)
+	while( 1)
 	{
-		ptyfd = ml_term_get_master_fd( terms[count]) ;
-		FD_SET( ptyfd , &read_fds) ;
+		maxfd = 0 ;
+		FD_ZERO( &read_fds) ;
 
-		if( ptyfd > maxfd)
+		for( count = 0 ; count < num_of_terms ; count ++)
 		{
-			maxfd = ptyfd ;
+			ptyfd = ml_term_get_master_fd( terms[count]) ;
+			FD_SET( ptyfd , &read_fds) ;
+
+			if( ptyfd > maxfd)
+			{
+				maxfd = ptyfd ;
+			}
+		}
+
+	#ifdef  USE_LIBSSH2
+		for( count = 0 ; count < num_of_xssh_fds ; count++)
+		{
+			FD_SET( xssh_fds[count] , &read_fds) ;
+
+			if( xssh_fds[count] > maxfd)
+			{
+				maxfd = xssh_fds[count] ;
+			}
+		}
+
+		tval.tv_usec = 0 ;
+		tval.tv_sec = keepalive_interval ;
+
+		if( select( maxfd + 1 , &read_fds , NULL , NULL ,
+				keepalive_interval > 0 ? &tval : NULL) == 0)
+		{
+			ml_pty_ssh_keepalive( keepalive_interval * 1000) ;
+		}
+		else
+	#else
+		select( maxfd + 1 , &read_fds , NULL , NULL , NULL) ;
+	#endif
+		{
+			break ;
 		}
 	}
 
-	select( maxfd + 1 , &read_fds , NULL , NULL , NULL) ;
+#ifdef  USE_LIBSSH2
+	for( count = 0 ; count < num_of_xssh_fds ; count++)
+	{
+		if( FD_ISSET( xssh_fds[count] , &read_fds))
+		{
+			ml_pty_ssh_send_recv_x11( count) ;
+		}
+	}
+#endif
 
 	return  JNI_TRUE ;
 #endif
@@ -1230,26 +1384,15 @@ Java_mlterm_MLTermPty_nativeRead(
 
 	if( nativeObj && nativeObj->term)
 	{
-		ml_line_t *  line ;
 		int  ret ;
 
 		/* For event listeners of ml_term_t. */
 		nativeObj->env = env ;
 		nativeObj->obj = obj ;
 
-		if( ( line = ml_term_get_cursor_line( nativeObj->term)))
-		{
-			ml_line_restore_color( line ,
-				ml_term_cursor_char_index( nativeObj->term)) ;
-		}
-
+		draw_cursor( nativeObj->term , ml_line_restore_color) ;
 		ret = ml_term_parse_vt100_sequence( nativeObj->term) ;
-
-		if( ( line = ml_term_get_cursor_line( nativeObj->term)))
-		{
-			ml_line_reverse_color( line ,
-				ml_term_cursor_char_index( nativeObj->term)) ;
-		}
+		draw_cursor( nativeObj->term , ml_line_reverse_color) ;
 
 		if( ret)
 		{
@@ -1329,27 +1472,32 @@ Java_mlterm_MLTermPty_nativeWrite(
 	kik_debug_printf( "WRITE TO PTY: %x" , (int)*str) ;
 #endif
 
+	/* In case local echo in ml_term_write(). */
+	draw_cursor( ((native_obj_t*)nativeObj)->term , ml_line_restore_color) ;
+
 	if( *str == '\0')
 	{
 		/* Control+space */
 		ml_term_write( ((native_obj_t*)nativeObj)->term , str , 1 , 0) ;
-
-		return  JNI_TRUE ;
 	}
-
-	(*utf8_parser->init)( utf8_parser) ;
-	(*utf8_parser->set_str)( utf8_parser , str , strlen( str)) ;
-	while( ! utf8_parser->is_eos &&
-	       ( len = ml_term_convert_to( ((native_obj_t*)nativeObj)->term ,
-				buf , sizeof(buf) , utf8_parser)) > 0)
+	else
 	{
-		ml_term_write( ((native_obj_t*)nativeObj)->term , buf , len , 0) ;
+		(*utf8_parser->init)( utf8_parser) ;
+		(*utf8_parser->set_str)( utf8_parser , str , strlen( str)) ;
+		while( ! utf8_parser->is_eos &&
+		       ( len = ml_term_convert_to( ((native_obj_t*)nativeObj)->term ,
+					buf , sizeof(buf) , utf8_parser)) > 0)
+		{
+			ml_term_write( ((native_obj_t*)nativeObj)->term , buf , len , 0) ;
+		}
+
+	#if  0
+		kik_debug_printf( " => DONE\n") ;
+	#endif
+		(*env)->ReleaseStringUTFChars( env , jstr , str) ;
 	}
 
-#if  0
-	kik_debug_printf( " => DONE\n") ;
-#endif
-	(*env)->ReleaseStringUTFChars( env , jstr , str) ;
+	draw_cursor( ((native_obj_t*)nativeObj)->term , ml_line_reverse_color) ;
 
 	return  JNI_TRUE ;
 }
@@ -1534,7 +1682,7 @@ Java_mlterm_MLTermPty_nativeGetRedrawString(
 				(*env)->SetIntField( env , styles[num_of_styles - 1] ,
 					style_fg_pixel ,
 					/* return -1(white) for invalid color. */
-					ml_get_color_rgb( color , &red , &green , &blue) ?
+					ml_get_color_rgba( color , &red , &green , &blue , NULL) ?
 						((red << 16) | (green << 8) | blue) : -1) ;
 
 				color = ml_char_bg_color( line->chars + mod_beg + count) ;
@@ -1543,7 +1691,7 @@ Java_mlterm_MLTermPty_nativeGetRedrawString(
 				(*env)->SetIntField( env , styles[num_of_styles - 1] ,
 					style_bg_pixel ,
 					/* return -1(white) for invalid color. */
-					ml_get_color_rgb( color , &red , &green , &blue) ?
+					ml_get_color_rgba( color , &red , &green , &blue , NULL) ?
 						((red << 16) | (green << 8) | blue) : -1) ;
 
 				(*env)->SetBooleanField( env , styles[num_of_styles - 1] ,
@@ -1968,17 +2116,62 @@ Java_mlterm_MLTermPty_getColorRGB(
 	jstring  jstr_color
 	)
 {
-	const char *  color ;
+	const char *  color_name ;
+	ml_color_t  color ;
 	u_int8_t  red ;
 	u_int8_t  green ;
 	u_int8_t  blue ;
-	int  ret ;
+	int  error ;
 
-	color = (*env)->GetStringUTFChars( env , jstr_color , NULL) ;
-	ret = ml_get_color_rgb( ml_get_color( color) , &red , &green , &blue) ;
-	(*env)->ReleaseStringUTFChars( env , jstr_color , color) ;
+	color_name = (*env)->GetStringUTFChars( env , jstr_color , NULL) ;
 
-	if( ret)
+	error = 0 ;
+
+	/*
+	 * The similar processing as that of x_load_named_xcolor()
+	 * in fb/x_color.c and win32/x_color.c.
+	 */
+
+	if( ml_color_parse_rgb_name( &red , &green , &blue , NULL , color_name))
+	{
+		/* do nothing */
+	}
+	else if( ( color = ml_get_color( color_name)) != ML_UNKNOWN_COLOR &&
+	         IS_VTSYS_BASE_COLOR(color))
+	{
+		/*
+		 * 0 : 0x00, 0x00, 0x00
+		 * 1 : 0xff, 0x00, 0x00
+		 * 2 : 0x00, 0xff, 0x00
+		 * 3 : 0xff, 0xff, 0x00
+		 * 4 : 0x00, 0x00, 0xff
+		 * 5 : 0xff, 0x00, 0xff
+		 * 6 : 0x00, 0xff, 0xff
+		 * 7 : 0xe5, 0xe5, 0xe5
+		 */
+		red = (color & 0x1) ? 0xff : 0 ;
+		green = (color & 0x2) ? 0xff : 0 ;
+		blue = (color & 0x4) ? 0xff : 0 ;
+	}
+	else
+	{
+		if( strcmp( color_name , "gray") == 0)
+		{
+			red = green = blue = 190 ;
+		}
+		else if( strcmp( color_name , "lightgray") == 0)
+		{
+			red = green = blue = 211 ;
+		}
+		else
+		{
+			error = 1 ;
+		}
+	}
+
+	(*env)->ReleaseStringUTFChars( env , jstr_color , color_name) ;
+
+	if( ! error)
 	{
 		return  ((red << 16) | (green << 8) | blue) ;
 	}
