@@ -6,6 +6,8 @@
 #define  __KIK_STR_H__
 
 
+#include  <string.h>		/* strlen/strsep/strdup */
+
 #include  "kik_types.h"		/* size_t */
 #include  "kik_def.h"
 #include  "kik_mem.h"		/* alloca */
@@ -13,8 +15,6 @@
 
 #ifdef  HAVE_STRSEP
 
-#include  <string.h>
-
 #define  kik_str_sep( strp , delim)  strsep( strp , delim)
 
 #else
@@ -36,10 +36,6 @@ int  kik_snprintf( char *  str , size_t  size , const char *  format , ...) ;
 
 #define  strdup( str)  kik_str_dup( str , __FILE__ , __LINE__ , __FUNCTION__)
 
-#else
-
-#include  <string.h>		/* strdup */
-
 #endif
 
 char *  kik_str_dup( const char *  str , const char *  file , int  line , const char *  func) ;
