@@ -15,7 +15,7 @@
 #include  <x_font.h>	/* x_use_cp932_ucs_fot_xft */
 #include  <x_screen_manager.h>
 #include  <x_event_source.h>
-#ifndef  USE_WIN32GUI
+#if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
 #include  <xlib/x_xim.h>
 #endif
 
@@ -103,6 +103,14 @@ get_font_size_range(
 	return  1 ;
 }
 
+#ifdef  USE_LIBSSH2
+static void
+ssh_keepalive(void)
+{
+	ml_pty_ssh_keepalive( 100) ;
+}
+#endif
+
 
 /* --- global functions --- */
 
@@ -115,7 +123,7 @@ main_loop_init(
 	x_main_config_t  main_config ;
 	kik_conf_t *  conf ;
 	char *  value ;
-#ifndef  USE_WIN32GUI
+#if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
 	int  use_xim ;
 #endif
 	u_int  max_screens_multiple ;
@@ -157,7 +165,7 @@ main_loop_init(
 		"show version message") ;
 	kik_conf_add_opt( conf , 'R' , "fsrange" , 0 , "font_size_range" , 
 		"font size range for GUI configurator [6-30]") ;
-#ifndef  USE_WIN32GUI
+#if  ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
 	kik_conf_add_opt( conf , 'Y' , "decsp" , 1 , "compose_dec_special_font" ,
 		"compose dec special font [false]") ;
 #endif
@@ -165,7 +173,7 @@ main_loop_init(
 	kik_conf_add_opt( conf , 'c' , "cp932" , 1 , "use_cp932_ucs_for_xft" , 
 		"use CP932-Unicode mapping table for JISX0208 [false]") ;
 #endif
-#ifndef  USE_WIN32GUI
+#if  ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
 	kik_conf_add_opt( conf , 'i' , "xim" , 1 , "use_xim" , 
 		"use XIM (X Input Method) [true]") ;
 #endif
@@ -183,6 +191,10 @@ main_loop_init(
 		"visual depth") ;
 	kik_conf_add_opt( conf , '\0' , "maxptys" , 0 , "max_ptys" ,
 		"max ptys to open simultaneously (multiple of 32)") ;
+#ifdef  USE_LIBSSH2
+	kik_conf_add_opt( conf , '\0' , "keepalive" , 0 , "ssh_keepalive_interval" ,
+		"interval seconds to send keepalive. [0 = not send]") ;
+#endif
 
 	if( ! kik_conf_parse_args( conf , &argc , &argv))
 	{
@@ -234,7 +246,7 @@ main_loop_init(
 	}
 #endif
 
-#ifndef  USE_WIN32GUI
+#if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
 	use_xim = 1 ;
 
 	if( ( value = kik_conf_get_value( conf , "use_xim")))
@@ -248,21 +260,7 @@ main_loop_init(
 	x_xim_init( use_xim) ;
 #endif
 
-	if( ( value = kik_conf_get_value( conf , "click_interval")))
-	{
-		int  interval ;
-
-		if( kik_str_to_int( &interval , value))
-		{
-			x_set_click_interval( interval) ;
-		}
-		else
-		{
-			kik_msg_printf( invalid_msg , "click_interval" , value) ;
-		}
-	}
-
-#ifndef  USE_WIN32GUI
+#if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
 	if( ( value = kik_conf_get_value( conf , "compose_dec_special_font")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -332,7 +330,36 @@ main_loop_init(
 			num_of_startup_screens = n ;
 		}
 	}
-	
+
+#ifdef  USE_LIBSSH2
+	if( ( value = kik_conf_get_value( conf , "ssh_keepalive_interval")))
+	{
+		u_int  interval ;
+
+		if( kik_str_to_uint( &interval , value) && interval > 0)
+		{
+			ml_pty_ssh_set_keepalive_interval( interval) ; 
+			x_event_source_add_fd( -1 , ssh_keepalive) ;
+		}
+	}
+#endif
+
+#if  defined(__NetBSD__) && defined(USE_FRAMEBUFFER)
+	if( ( value = kik_conf_get_value( conf , "wskbd_repeat_1")))
+	{
+		extern int  wskbd_repeat_1 ;
+
+		kik_str_to_int( &wskbd_repeat_1 , value) ;
+	}
+
+	if( ( value = kik_conf_get_value( conf , "wskbd_repeat_N")))
+	{
+		extern int  wskbd_repeat_N ;
+
+		kik_str_to_int( &wskbd_repeat_N , value) ;
+	}
+#endif
+
 	x_main_config_init( &main_config , conf , argc , argv) ;
 
 	kik_conf_delete( conf) ;
@@ -369,7 +396,7 @@ main_loop_final(void)
 
 	kik_set_msg_log_file_name( NULL) ;
 
-#ifndef  USE_WIN32GUI
+#if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
 	x_xim_final() ;
 #endif
 
@@ -391,7 +418,7 @@ main_loop_start(void)
 
 		if( ! is_genuine_daemon)
 		{
-			kik_msg_printf( "Unable to start - open_screen_intern() failed.\n") ;
+			kik_msg_printf( "Unable to open screen.\n") ;
 
 			return  0 ;
 		}
