@@ -10,7 +10,6 @@
 
 
 #define  CHARSET(attr)  (((attr) >> 7) & 0x1ff)
-#define  SIZE(attr) (CS_SIZE(CHARSET(attr)))
 
 #define  IS_BIWIDTH(attr)  ((attr) & (0x1 << 6))
 
@@ -63,6 +62,7 @@ get_comb_size(
 	return  size ;
 }
 
+/* See also ml_get_closest_color() in ml_color.c. */
 inline static u_int8_t
 intern_color(
 	ml_color_t  ex_color
@@ -190,7 +190,8 @@ ml_char_set(
 	)
 {
 #ifdef  DEBUG
-	if( CS_SIZE(cs) != size)
+	/* Ignore PICTURE_CHARSET. */
+	if( cs != PICTURE_CHARSET && CS_SIZE(cs) != size)
 	{
 		kik_debug_printf( KIK_DEBUG_TAG " char size error => size %d <=> CS_SIZE %d\n",
 			size, CS_SIZE(cs)) ;
@@ -217,25 +218,9 @@ ml_char_set(
 	{
 		kik_warn_printf(KIK_DEBUG_TAG "charset(%x) is out of range\n", cs) ;
 	}
-	if( is_biwidth > 1 || is_biwidth < 0)
-	{
-		kik_warn_printf(KIK_DEBUG_TAG "is_biwidth should be 0/1\n") ;
-	}
-	if( is_comb > 1 || is_comb < 0)
-	{
-		kik_warn_printf(KIK_DEBUG_TAG "is_comb should be 0/1\n") ;
-	}
-	if( is_bold > 1 || is_bold < 0)
-	{
-		kik_warn_printf(KIK_DEBUG_TAG "is_bold should be 0/1\n") ;
-	}
-	if( is_underlined > 1 || is_underlined < 0)
-	{
-		kik_warn_printf(KIK_DEBUG_TAG "is_underlined should be 0/1\n") ;
-	}
 #endif
 
-	ch->u.ch.attr = COMPOUND_ATTR(cs,is_biwidth,is_bold,is_underlined,is_comb) ;
+	ch->u.ch.attr = COMPOUND_ATTR(cs,is_biwidth!=0,is_bold!=0,is_underlined!=0,is_comb!=0) ;
 	ch->u.ch.fg_color = intern_color(fg_color) ;
 	ch->u.ch.bg_color = intern_color(bg_color) ;
 
@@ -275,6 +260,13 @@ ml_char_combine(
 	}
 #endif
 
+#ifdef  DEBUG
+	if( cs >= MAX_CHARSET)
+	{
+		kik_warn_printf(KIK_DEBUG_TAG "charset(%x) is out of range\n", cs) ;
+	}
+#endif
+
 	if( IS_SINGLE_CH(ch->u.ch.attr))
 	{
 		if( ( multi_ch = malloc( sizeof( ml_char_t) * 2)) == NULL)
@@ -328,7 +320,7 @@ ml_char_combine(
 			return  0 ;
 		}
 
-	#if !defined(__GLIBC__)
+	#ifndef  __GLIBC__
 		if( sizeof( multi_ch) >= 8 && ((long)( multi_ch) & 0x1UL) != 0)
 		{
 			kik_msg_printf( "Your malloc() doesn't return 2 bits aligned address."
@@ -359,8 +351,10 @@ ml_char_combine_simple(
 	ml_char_t *  comb
 	)
 {
-	return  ml_char_combine( ch , ml_char_bytes( comb) , SIZE(comb->u.ch.attr) ,
-			CHARSET(comb->u.ch.attr) ,
+	mkf_charset_t  cs ;
+
+	cs = CHARSET(comb->u.ch.attr) ;
+	return  ml_char_combine( ch , ml_char_bytes( comb) , CS_SIZE(cs) , cs ,
 			IS_BIWIDTH(comb->u.ch.attr) , IS_COMB(comb->u.ch.attr) ,
 			comb->u.ch.fg_color , comb->u.ch.bg_color ,
 			IS_BOLD(comb->u.ch.attr) , IS_UNDERLINED(comb->u.ch.attr)) ;
@@ -491,7 +485,11 @@ ml_char_size(
 {
 	if( IS_SINGLE_CH(ch->u.ch.attr))
 	{
-		return  SIZE(ch->u.ch.attr) ;
+		mkf_charset_t  cs ;
+
+		cs = CHARSET(ch->u.ch.attr) ;
+
+		return  CS_SIZE(cs) ;
 	}
 	else
 	{
@@ -507,7 +505,11 @@ ml_char_set_bytes(
 {
 	if( IS_SINGLE_CH(ch->u.ch.attr))
 	{
-		memcpy( ch->u.ch.bytes , bytes , SIZE(ch->u.ch.attr)) ;
+		mkf_charset_t  cs ;
+
+		cs = CHARSET(ch->u.ch.attr) ;
+
+		memcpy( ch->u.ch.bytes , bytes , CS_SIZE(cs)) ;
 	}
 	else
 	{
@@ -537,23 +539,15 @@ ml_char_font(
 	ml_char_t *  ch
 	)
 {
-	if( IS_SINGLE_CH(ch->u.ch.attr))
+	u_int  attr ;
+
+	attr = ch->u.ch.attr ;
+
+	if( IS_SINGLE_CH(attr))
 	{
-		ml_font_t  font ;
-
-		font = CHARSET(ch->u.ch.attr) ;
-
-		if( IS_BOLD(ch->u.ch.attr))
-		{
-			font |= FONT_BOLD ;
-		}
-
-		if( IS_BIWIDTH(ch->u.ch.attr))
-		{
-			font |= FONT_BIWIDTH ;
-		}
-
-		return  font ;
+		return  CHARSET(attr) |
+		        (IS_BOLD(attr) ? FONT_BOLD : 0) |
+		        (IS_BIWIDTH(attr) ? FONT_BIWIDTH : 0) ;
 	}
 	else
 	{
@@ -561,6 +555,10 @@ ml_char_font(
 	}
 }
 
+/*
+ * Return the number of columns when ch is shown in the screen.
+ * (If ml_char_cols(ch) returns 0, nothing is shown in the screen.)
+ */
 u_int
 ml_char_cols(
 	ml_char_t *  ch
@@ -570,6 +568,7 @@ ml_char_cols(
 	{
 		return  2 ;
 	}
+#if  1
 	else if( IS_SINGLE_CH(ch->u.ch.attr) && CHARSET(ch->u.ch.attr) == ISO10646_UCS4_1)
 	{
 		/*
@@ -585,14 +584,19 @@ ml_char_cols(
 		 * 202D;LEFT-TO-RIGHT OVERRIDE
 		 * 202E;RIGHT-TO-LEFT OVERRIDE
 		*/
-		if( ch->u.ch.bytes[2] == 0x20 &&
-		    ((0x0c <= ch->u.ch.bytes[3] && ch->u.ch.bytes[3] <= 0x0f) ||
-		     (0x2a <= ch->u.ch.bytes[3] && ch->u.ch.bytes[3] <= 0x2e)) &&
-		    ch->u.ch.bytes[0] == 0 && ch->u.ch.bytes[1] == 0)
+		u_char *  bytes ;
+
+		bytes = ch->u.ch.bytes ;
+
+		if( bytes[2] == 0x20 &&
+		    ((0x0c <= bytes[3] && bytes[3] <= 0x0f) ||
+		     (0x2a <= bytes[3] && bytes[3] <= 0x2e)) &&
+		    bytes[0] == 0 && bytes[1] == 0)
 		{
 			return  0 ;
 		}
 	}
+#endif
 
 	return  1 ;
 }
@@ -635,11 +639,15 @@ ml_char_fg_color(
 	ml_char_t *  ch
 	)
 {
-	if( IS_SINGLE_CH(ch->u.ch.attr))
+	u_int  attr ;
+
+	attr = ch->u.ch.attr ;
+
+	if( IS_SINGLE_CH(attr))
 	{
 		ml_color_t  color ;
 
-		if( IS_REVERSED(ch->u.ch.attr))
+		if( IS_REVERSED(attr))
 		{
 			color = extern_color( ch->u.ch.bg_color) ;
 
@@ -651,7 +659,7 @@ ml_char_fg_color(
 		{
 			color = extern_color( ch->u.ch.fg_color) ;
 
-			if( color < MAX_VTSYS_COLORS && IS_BOLD(ch->u.ch.attr))
+			if( color < MAX_VTSYS_COLORS && IS_BOLD(attr))
 			{
 				color |= ML_BOLD_COLOR_MASK ;
 			}
@@ -697,15 +705,19 @@ ml_char_bg_color(
 	ml_char_t *  ch
 	)
 {
-	if( IS_SINGLE_CH(ch->u.ch.attr))
+	u_int  attr ;
+
+	attr = ch->u.ch.attr ;
+
+	if( IS_SINGLE_CH(attr))
 	{
 		ml_color_t  color ;
 
-		if( IS_REVERSED(ch->u.ch.attr))
+		if( IS_REVERSED(attr))
 		{
 			color = extern_color(ch->u.ch.fg_color) ;
 
-			if( color < MAX_VTSYS_COLORS && IS_BOLD(ch->u.ch.attr))
+			if( color < MAX_VTSYS_COLORS && IS_BOLD(attr))
 			{
 				color |= ML_BOLD_COLOR_MASK ;
 			}
@@ -870,7 +882,11 @@ ml_char_is_null(
 {
 	if( IS_SINGLE_CH( ch->u.ch.attr))
 	{
-		return  (SIZE(ch->u.ch.attr) == 1 && ch->u.ch.bytes[0] == '\0') ;
+		mkf_charset_t  cs ;
+
+		cs = CHARSET(ch->u.ch.attr) ;
+
+		return  (CS_SIZE(cs) == 1 && ch->u.ch.bytes[0] == '\0') ;
 	}
 	else
 	{
@@ -904,8 +920,8 @@ ml_char_bytes_is(
 	if( IS_SINGLE_CH(ch->u.ch.attr))
 	{
 		if( CHARSET(ch->u.ch.attr) == cs &&
-			SIZE(ch->u.ch.attr) == size &&
-			memcmp( ml_char_bytes( ch) , bytes , size) == 0)
+		    CS_SIZE(cs) == size &&
+		    memcmp( ml_char_bytes( ch) , bytes , size) == 0)
 		{
 			return  1 ;
 		}
@@ -999,7 +1015,7 @@ ml_nl_ch(void)
 
 #ifdef  DEBUG
 
-#if  0
+#if  1
 #define  DUMP_HEX
 #endif
 
