@@ -31,6 +31,10 @@
 #define  MLCHAR_XCT_MAX_SIZE  (XCT_MAX_SIZE * (MAX_COMB_SIZE + 1))
 
 
+/* For inline pictures (see x_picture.c) */
+#define  PICTURE_CHARSET  0x1ff
+
+
 /*
  * This object size should be kept as small as possible.
  * (ILP32: 64bit) (LP64: 64bit)
@@ -103,8 +107,6 @@ int  ml_char_combine( ml_char_t *  ch , u_char *  bytes , size_t  size ,
 
 int  ml_char_combine_simple( ml_char_t *  ch , ml_char_t *  comb) ;
 
-int  ml_remove_combining_char( ml_char_t *  ch) ;
-
 ml_char_t *  ml_get_base_char( ml_char_t *  ch) ;
 
 ml_char_t *  ml_get_combining_chars( ml_char_t *  ch , u_int *  size) ;
