@@ -323,6 +323,54 @@ ml_edit_final(
 	return  1 ;
 }
 
+int
+ml_edit_clone(
+	ml_edit_t *  dst_edit ,
+	ml_edit_t *  src_edit
+	)
+{
+	u_int  row ;
+	u_int  num_of_rows ;
+	ml_line_t *  src_line ;
+	ml_line_t *  dst_line ;
+
+	memcpy( ((char*)dst_edit) + sizeof(ml_model_t) ,
+		((char*)src_edit) + sizeof(ml_model_t) ,
+		sizeof(ml_edit_t) - sizeof(ml_model_t)) ;
+
+	if( ! ( dst_edit->tab_stops = malloc( TAB_STOPS_SIZE(src_edit))))
+	{
+		return  0 ;
+	}
+	memcpy( dst_edit->tab_stops , src_edit->tab_stops , TAB_STOPS_SIZE(src_edit)) ;
+
+	dst_edit->cursor.model = &dst_edit->model ;
+
+	num_of_rows = ml_edit_get_rows( src_edit) ;
+
+	if( ! ml_model_init( &dst_edit->model , ml_edit_get_cols( src_edit) , num_of_rows))
+	{
+		free( dst_edit->tab_stops) ;
+
+		return  0 ;
+	}
+
+	for( row = 0 ; row < num_of_rows ; row++)
+	{
+		dst_line = ml_edit_get_line( dst_edit , row) ;
+
+		if( ( src_line = ml_edit_get_line( src_edit , row)) ==
+		    src_edit->wraparound_ready_line)
+		{
+			dst_edit->wraparound_ready_line = dst_line ;
+		}
+
+		ml_line_copy_line( dst_line , src_line) ;
+	}
+
+	return  1 ;
+}
+
 int
 ml_edit_resize(
 	ml_edit_t *  edit ,
@@ -821,9 +869,7 @@ ml_edit_clear_line_to_right(
 
 	if( edit->use_bce)
 	{
-		ml_line_fill( cursor_line , &edit->bce_ch , edit->cursor.char_index ,
-			edit->model.num_of_cols -
-				ml_str_cols( cursor_line->chars , edit->cursor.char_index)) ;
+		ml_line_clear_with( cursor_line , edit->cursor.char_index , &edit->bce_ch) ;
 	}
 	else
 	{
@@ -870,8 +916,8 @@ ml_edit_clear_below(
 
 		for( row = edit->cursor.row + 1 ; row < edit->model.num_of_rows ; row ++)
 		{
-			ml_line_fill( ml_model_get_line( &edit->model , row) , &edit->bce_ch , 0 ,
-				edit->model.num_of_cols) ;
+			ml_line_clear_with( ml_model_get_line( &edit->model , row) ,
+				0 , &edit->bce_ch) ;
 		}
 
 		return  1 ;
@@ -901,8 +947,8 @@ ml_edit_clear_above(
 
 		for( row = 0 ; row < edit->cursor.row ; row ++)
 		{
-			ml_line_fill( ml_model_get_line( &edit->model , row) , &edit->bce_ch , 0 ,
-				edit->model.num_of_cols) ;
+			ml_line_clear_with( ml_model_get_line( &edit->model , row) ,
+				0 , &edit->bce_ch) ;
 		}
 
 		return  1 ;
@@ -920,11 +966,10 @@ ml_edit_fill_all(
 	)
 {
 	int  row ;
-	
+
 	for( row = 0 ; row < edit->model.num_of_rows ; row ++)
 	{
-		ml_line_fill( ml_model_get_line( &edit->model , row) , ch , 0 ,
-			edit->model.num_of_cols / ml_char_cols(ch)) ;
+		ml_line_clear_with( ml_model_get_line( &edit->model , row) , 0 , ch) ;
 	}
 
 	return  1 ;
@@ -1490,6 +1535,185 @@ ml_edit_restore_cursor(
 	return  ml_cursor_restore( &edit->cursor) ;
 }
 
+int
+ml_edit_copy_area(
+	ml_edit_t *  edit ,
+	int  src_col ,
+	int  src_row ,
+	u_int  num_of_copy_cols ,
+	u_int  num_of_copy_rows ,
+	int  dst_col ,
+	int  dst_row
+	)
+{
+	u_int  count ;
+	ml_line_t *  src_line ;
+	ml_line_t *  dst_line ;
+	int  src_char_index ;
+	int  dst_char_index ;
+	u_int  src_cols_rest ;
+	u_int  src_cols_after ;
+	u_int  dst_cols_rest ;
+	u_int  num_of_src_chars ;
+	u_int  num_of_src_cols ;
+
+	if( edit->is_relative_origin)
+	{
+		if( (src_row += edit->scroll_region_beg) > edit->scroll_region_end ||
+		    (dst_row += edit->scroll_region_beg) > edit->scroll_region_end)
+		{
+			return  1 ;
+		}
+
+		if( src_row + num_of_copy_rows > edit->scroll_region_end + 1)
+		{
+			num_of_copy_rows = edit->scroll_region_end + 1 - src_row ;
+		}
+
+		if( dst_row + num_of_copy_rows > edit->scroll_region_end + 1)
+		{
+			num_of_copy_rows = edit->scroll_region_end + 1 - dst_row ;
+		}
+	}
+
+	for( count = 0 ; count < num_of_copy_rows ; count++)
+	{
+		int  srow ;
+		int  drow ;
+
+		if( src_row < dst_row)
+		{
+			srow = src_row + num_of_copy_rows - count - 1 ;
+			drow = dst_row + num_of_copy_rows - count - 1 ;
+		}
+		else
+		{
+			srow = src_row + count ;
+			drow = dst_row + count ;
+		}
+
+		if( ! (src_line = ml_edit_get_line( edit , srow)) ||
+		    ! (dst_line = ml_edit_get_line( edit , drow)))
+		{
+			continue ;
+		}
+
+		src_char_index = ml_convert_col_to_char_index( src_line ,
+					&src_cols_rest , src_col , BREAK_BOUNDARY) ;
+		if( src_cols_rest > 0)
+		{
+			src_cols_after = ml_char_cols( src_line->chars + src_char_index) -
+						src_cols_rest ;
+			src_char_index ++ ;
+		}
+		else
+		{
+			src_cols_after = 0 ;
+		}
+
+		dst_char_index = ml_convert_col_to_char_index( dst_line ,
+					&dst_cols_rest , dst_col , 0) ;
+
+		if( dst_cols_rest + src_cols_rest > 0)
+		{
+			ml_line_fill( dst_line , ml_sp_ch() , dst_char_index ,
+				dst_cols_rest + src_cols_after) ;
+			dst_char_index += (dst_cols_rest + src_cols_after) ;
+		}
+
+		if( src_line->num_of_filled_chars > src_char_index)
+		{
+			num_of_src_chars = ml_convert_col_to_char_index( src_line ,
+						&src_cols_rest ,
+						src_col + num_of_copy_cols , 0)
+						- src_char_index ;
+			num_of_src_cols = num_of_copy_cols - src_cols_rest ;
+		}
+		else
+		{
+			num_of_src_chars = 0 ;
+			num_of_src_cols = 0 ;
+		}
+
+		if( num_of_src_cols > 0)
+		{
+			ml_line_overwrite( dst_line , dst_char_index ,
+				src_line->chars + src_char_index ,
+				num_of_src_chars , num_of_src_cols - src_cols_after) ;
+		}
+
+		if( num_of_src_cols + src_cols_after < num_of_copy_cols)
+		{
+			ml_line_fill( dst_line , ml_sp_ch() ,
+				dst_char_index + num_of_src_chars ,
+				num_of_copy_cols - num_of_src_cols) ;
+		}
+	}
+
+	return  1 ;
+}
+
+int
+ml_edit_erase_area(
+	ml_edit_t *  edit ,
+	int  col ,
+	int  row ,
+	u_int  num_of_cols ,
+	u_int  num_of_rows
+	)
+{
+	u_int  count ;
+	ml_line_t *  line ;
+	int  char_index ;
+	u_int  cols_rest ;
+
+	if( edit->is_relative_origin)
+	{
+		if( (row += edit->scroll_region_beg) > edit->scroll_region_end)
+		{
+			return  1 ;
+		}
+
+		if( row + num_of_rows > edit->scroll_region_end + 1)
+		{
+			num_of_rows = edit->scroll_region_end + 1 - row ;
+		}
+	}
+
+	for( count = 0 ; count < num_of_rows ; count++)
+	{
+		if( ! (line = ml_edit_get_line( edit , row + count)))
+		{
+			continue ;
+		}
+
+		char_index = ml_convert_col_to_char_index( line , &cols_rest , col ,
+					BREAK_BOUNDARY) ;
+
+		if( char_index >= line->num_of_filled_chars)
+		{
+			if( ! edit->use_bce)
+			{
+				continue ;
+			}
+
+			ml_line_fill( line , ml_sp_ch() , char_index ,
+					char_index + 1 - line->num_of_filled_chars) ;
+		}
+
+		if( cols_rest > 0)
+		{
+			ml_line_fill( line , ml_sp_ch() , char_index , cols_rest) ;
+			char_index += cols_rest ;
+		}
+
+		ml_line_fill( line , edit->use_bce ? &edit->bce_ch : ml_sp_ch() ,
+				char_index , num_of_cols) ;
+	}
+
+	return  1 ;
+}
+
 
 /*
  * for debugging.
