@@ -195,6 +195,14 @@ ml_iscii_reset(
 #define  ml_bidi_reset( state)  (0)
 #else
 /* Link functions in libctl/ml_*bidi.c */
+int  ml_line_set_use_bidi( ml_line_t *  line , int  flag) ;
+int  ml_line_bidi_convert_visual_char_index_to_logical( ml_line_t *  line , int  char_index) ;
+int  ml_line_bidi_copy_logical_str( ml_line_t *  line , ml_char_t *  dst , int  beg , u_int  len) ;
+int  ml_line_bidi_is_rtl( ml_line_t *  line) ;
+int  ml_bidi_copy( ml_bidi_state_t  dst , ml_bidi_state_t  src) ;
+int  ml_bidi_reset( ml_bidi_state_t  state) ;
+int  ml_line_bidi_convert_logical_char_index_to_visual( ml_line_t *  line , int  char_index ,
+	int *  ltr_rtl_meet_pos) ;
 #endif
 
 #ifndef  USE_IND
@@ -203,9 +211,14 @@ ml_iscii_reset(
 #define  ml_iscii_reset( state)  (0)
 #else
 /* Link functions in libctl/ml_*iscii.c */
+int  ml_line_set_use_iscii( ml_line_t *  line , int  flag) ;
+int  ml_iscii_copy( ml_iscii_state_t  dst , ml_iscii_state_t  src) ;
+int  ml_iscii_reset( ml_iscii_state_t  state) ;
+int  ml_line_iscii_convert_logical_char_index_to_visual( ml_line_t *  line ,
+	int  logical_char_index) ;
 #endif
 
-#endif
+#endif	/* NO_DYNAMIC_LOAD_CTL */
 
 
 /* --- global functions --- */
@@ -429,10 +442,25 @@ ml_line_clear(
 
 	ml_char_copy( line->chars + char_index , ml_sp_ch()) ;
 	line->num_of_filled_chars = char_index + 1 ;
-	
+	line->is_continued_to_next = 0 ;
+
 	return  1 ;
 }
 
+int
+ml_line_clear_with(
+	ml_line_t *  line ,
+	int  char_index ,
+	ml_char_t *  ch
+	)
+{
+	line->is_continued_to_next = 0 ;
+
+	return  ml_line_fill( line , ch , char_index ,
+			(line->num_of_chars - ml_str_cols( line->chars , char_index))
+			/ ml_char_cols(ch)) ;
+}
+
 int
 ml_line_overwrite(
 	ml_line_t *  line ,
