@@ -137,8 +137,8 @@ scrolled_out_lines_finished(
 	ml_screen_t *  screen ;
 
 	screen = p ;
-	
-	if( ml_screen_is_backscrolling( screen) == BSM_VOLATILE)
+
+	if( ml_screen_is_backscrolling( screen) == BSM_DEFAULT)
 	{
 		ml_screen_set_modified_all( screen) ;
 	}
@@ -159,7 +159,7 @@ window_scroll_upward_region(
 	if( screen->is_backscrolling)
 	{
 		/*
-		 * Not necessary to scrolling window. If backscroll_mode is BSM_VOLATILE,
+		 * Not necessary to scrolling window. If backscroll_mode is BSM_DEFAULT,
 		 * ml_screen_set_modified_all() in scrolled_out_lines_finished() later.
 		 */
 		return  1 ;
@@ -190,7 +190,7 @@ window_scroll_downward_region(
 	if( screen->is_backscrolling)
 	{
 		/*
-		 * Not necessary to scrolling window. If backscroll_mode is BSM_VOLATILE,
+		 * Not necessary to scrolling window. If backscroll_mode is BSM_DEFAULT,
 		 * ml_screen_set_modified_all() in scrolled_out_lines_finished() later.
 		 */
 		return  1 ;
@@ -1050,17 +1050,34 @@ ml_screen_logical(
 	}
 }
 
+ml_bs_mode_t
+ml_screen_is_backscrolling(
+	ml_screen_t *  screen
+	)
+{
+	if( screen->is_backscrolling == BSM_STATIC)
+	{
+		if( screen->backscroll_rows < ml_get_log_size( &screen->logs))
+		{
+			return  BSM_STATIC ;
+		}
+		else
+		{
+			return  BSM_DEFAULT ;
+		}
+	}
+	else
+	{
+		return  screen->is_backscrolling ;
+	}
+}
+
 int
 ml_set_backscroll_mode(
 	ml_screen_t *  screen ,
 	ml_bs_mode_t   mode
 	)
 {
-	if( mode != BSM_VOLATILE && mode != BSM_STATIC)
-	{
-		return  0 ;
-	}
-	
 	screen->backscroll_mode = mode ;
 
 	if( screen->is_backscrolling)
@@ -1489,378 +1506,6 @@ ml_screen_get_word_region(
 	return  1 ;
 }
 
-
-/*
- * VT100 commands
- *
- * !! Notice !!
- * These functions are called under logical order mode.
- */
-
-ml_char_t *
-ml_screen_get_n_prev_char(
-	ml_screen_t *  screen ,
-	int  n
-	)
-{
-	int  char_index ;
-	int  row ;
-	ml_char_t *  ch ;
-	ml_line_t *  line ;
-
-	if( ! get_n_prev_char_pos( screen , &char_index , &row , 1))
-	{
-		return  NULL ;
-	}
-
-	if( ( line = ml_edit_get_line( screen->edit , row)) == NULL)
-	{
-		return  NULL ;
-	}
-
-	if( ( ch = ml_char_at( line , char_index)) == NULL)
-	{
-		return  NULL ;
-	}
-
-	return  ch ;
-}
-
-int
-ml_screen_combine_with_prev_char(
-	ml_screen_t *  screen ,
-	u_char *  bytes ,
-	size_t  ch_size ,
-	mkf_charset_t  cs ,
-	int  is_biwidth ,
-	int  is_comb ,
-	ml_color_t  fg_color ,
-	ml_color_t  bg_color ,
-	int  is_bold ,
-	int  is_underlined
-	)
-{
-	int  char_index ;
-	int  row ;
-	ml_char_t *  ch ;
-	ml_line_t *  line ;
-
-	if( ! get_n_prev_char_pos( screen , &char_index , &row , 1))
-	{
-		return  0 ;
-	}
-
-	if( ( line = ml_edit_get_line( screen->edit , row)) == NULL)
-	{
-		return  0 ;
-	}
-
-	if( ( ch = ml_char_at( line , char_index)) == NULL)
-	{
-		return  0 ;
-	}
-	
-	if( ! ml_char_combine( ch , bytes , ch_size , cs , is_biwidth , is_comb ,
-		fg_color , bg_color , is_bold , is_underlined))
-	{
-		return  0 ;
-	}
-	
-	ml_line_set_modified( line , char_index , char_index) ;
-	
-	return  1 ;
-}
-
-int
-ml_screen_insert_chars(
-	ml_screen_t *  screen ,
-	ml_char_t *  chars ,
-	u_int  len
-	)
-{
-	return  ml_edit_insert_chars( screen->edit , chars , len) ;
-}
-
-int
-ml_screen_insert_new_lines(
-	ml_screen_t *  screen ,
-	u_int  size
-	)
-{
-	u_int  count ;
-
-	for( count = 0 ; count < size ; count ++)
-	{
-		ml_edit_insert_new_line( screen->edit) ;
-	}
-
-	return  1 ;
-}
-
-int
-ml_screen_overwrite_chars(
-	ml_screen_t *  screen ,
-	ml_char_t *  chars ,
-	u_int  len
-	)
-{
-	return  ml_edit_overwrite_chars( screen->edit , chars , len) ;
-}
-
-int
-ml_screen_delete_lines(
-	ml_screen_t *  screen ,
-	u_int  size
-	)
-{
-	u_int  count ;
-
-	for( count = 0 ; count < size ; count ++)
-	{
-		if( ! ml_edit_delete_line( screen->edit))
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " deleting nonexisting lines.\n") ;
-		#endif
-			
-			return  0 ;
-		}
-	}
-	
-	return  1 ;
-}
-
-int
-ml_screen_go_forward(
-	ml_screen_t *  screen ,
-	u_int  size
-	)
-{
-	u_int  count ;
-
-	for( count = 0 ; count < size ; count ++)
-	{
-		if( ! ml_edit_go_forward( screen->edit , 0))
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " cursor cannot go forward any more.\n") ;
-		#endif
-		
-			return  0 ;
-		}
-	}
-
-	return  1 ;
-}
-
-int
-ml_screen_go_back(
-	ml_screen_t *  screen ,
-	u_int  size
-	)
-{
-	u_int  count ;
-
-	for( count = 0 ; count < size ; count ++)
-	{
-		if( ! ml_edit_go_back( screen->edit , 0))
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " cursor cannot go back any more.\n") ;
-		#endif
-					
-			return  0 ;
-		}
-	}
-
-	return  1 ;
-}
-
-int
-ml_screen_go_upward(
-	ml_screen_t *  screen ,
-	u_int  size
-	)
-{
-	u_int  count ;
-	
-	for( count = 0 ; count < size ; count ++)
-	{
-		if( ! ml_edit_go_upward( screen->edit , 0))
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " cursor cannot go upward any more.\n") ;
-		#endif
-				
-			return  0 ;
-		}
-	}
-
-	return  1 ;
-}
-
-int
-ml_screen_go_downward(
-	ml_screen_t *  screen ,
-	u_int  size
-	)
-{
-	u_int  count ;
-	
-	for( count = 0 ; count < size ; count ++)
-	{
-		if( ! ml_edit_go_downward( screen->edit , 0))
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " cursor cannot go downward any more.\n") ;
-		#endif
-		
-			return  0 ;
-		}
-	}
-
-	return  1 ;
-}
-
-int
-ml_screen_go_horizontally(
-	ml_screen_t *  screen ,
-	int  col
-	)
-{
-	return  ml_screen_goto( screen , col , ml_cursor_row( screen->edit)) ;
-}
-
-int
-ml_screen_go_vertically(
-	ml_screen_t *  screen ,
-	int  row
-	)
-{
-	return  ml_screen_goto( screen , ml_cursor_col( screen->edit) , row) ;
-}
-
-int
-ml_screen_cursor_visible(
-	ml_screen_t *  screen
-	)
-{
-	if( screen->is_cursor_visible)
-	{
-		return  1 ;
-	}
-	
-	screen->is_cursor_visible = 1 ;
-
-	return  1 ;
-}
-
-int
-ml_screen_cursor_invisible(
-	ml_screen_t *  screen
-	)
-{
-	if( ! screen->is_cursor_visible)
-	{
-		return  1 ;
-	}
-	
-	screen->is_cursor_visible = 0 ;
-
-	return  1 ;
-}
-
-int
-ml_screen_is_alternative_edit(
-	ml_screen_t *  screen
-        )
-{
-	if( screen->edit == &(screen->alt_edit) )
-	{
-		return  1 ;
-	}
-
-	return  0 ;
-}
-
-int
-ml_screen_use_normal_edit(
-	ml_screen_t *  screen
-	)
-{
-	screen->edit = &screen->normal_edit ;
-
-	if( screen->logvis)
-	{
-		(*screen->logvis->init)( screen->logvis ,
-			&screen->edit->model , &screen->edit->cursor) ;
-	}
-
-	ml_edit_set_modified_all( screen->edit) ;
-	
-	return  1 ;
-}
-
-int
-ml_screen_use_alternative_edit(
-	ml_screen_t *  screen
-	)
-{
-	screen->edit = &screen->alt_edit ;
-
-	ml_screen_goto_home( screen) ;
-	ml_screen_clear_below( screen) ;
-
-	if( screen->logvis)
-	{
-		(*screen->logvis->init)( screen->logvis ,
-			&screen->edit->model , &screen->edit->cursor) ;
-	}
-
-	ml_edit_set_modified_all( screen->edit) ;
-	
-	return  1 ;
-}
-
-int
-ml_screen_fill_all_with_e(
-	ml_screen_t *  screen
-	)
-{
-	ml_char_t  e_ch ;
-
-	ml_char_init( &e_ch) ;
-	ml_char_set( &e_ch , "E" , 1 , US_ASCII , 0 , 0 , ML_FG_COLOR , ML_BG_COLOR , 0 , 0) ;
-
-	ml_edit_fill_all( screen->edit , &e_ch) ;
-
-	ml_char_final( &e_ch) ;
-
-	return  1 ;
-}
-
-ml_bs_mode_t
-ml_screen_is_backscrolling(
-	ml_screen_t *  screen
-	)
-{
-	if( screen->is_backscrolling == BSM_STATIC)
-	{
-		if( screen->backscroll_rows < ml_get_log_size( &screen->logs))
-		{
-			return  BSM_STATIC ;
-		}
-		else
-		{
-			return  BSM_VOLATILE ;
-		}
-	}
-	else
-	{
-		return  screen->is_backscrolling ;
-	}
-}
-
 int
 ml_screen_search_init(
 	ml_screen_t *  screen ,
@@ -1958,7 +1603,7 @@ ml_screen_search_find(
 	{
 		return  0 ;
 	}
-	
+
 	if( ! (conv = ml_conv_new( ML_UTF8)))
 	{
 		(*parser->delete)( parser) ;
@@ -2127,7 +1772,7 @@ ml_screen_search_find(
 					line , *beg_char_index , &meet_pos) ;
 			end = ml_line_convert_logical_char_index_to_visual(
 					line , *end_char_index , &meet_pos) ;
-			
+
 			if( beg > end)
 			{
 				*beg_char_index = end ;
@@ -2161,3 +1806,476 @@ ml_screen_search_find(
 
 	return  res ;
 }
+
+
+/*
+ * VT100 commands
+ *
+ * !! Notice !!
+ * These functions are called under logical order mode.
+ */
+
+ml_char_t *
+ml_screen_get_n_prev_char(
+	ml_screen_t *  screen ,
+	int  n
+	)
+{
+	int  char_index ;
+	int  row ;
+	ml_char_t *  ch ;
+	ml_line_t *  line ;
+
+	if( ! get_n_prev_char_pos( screen , &char_index , &row , 1))
+	{
+		return  NULL ;
+	}
+
+	if( ( line = ml_edit_get_line( screen->edit , row)) == NULL)
+	{
+		return  NULL ;
+	}
+
+	if( ( ch = ml_char_at( line , char_index)) == NULL)
+	{
+		return  NULL ;
+	}
+
+	return  ch ;
+}
+
+int
+ml_screen_combine_with_prev_char(
+	ml_screen_t *  screen ,
+	u_char *  bytes ,
+	size_t  ch_size ,
+	mkf_charset_t  cs ,
+	int  is_biwidth ,
+	int  is_comb ,
+	ml_color_t  fg_color ,
+	ml_color_t  bg_color ,
+	int  is_bold ,
+	int  is_underlined
+	)
+{
+	int  char_index ;
+	int  row ;
+	ml_char_t *  ch ;
+	ml_line_t *  line ;
+
+	if( ! get_n_prev_char_pos( screen , &char_index , &row , 1))
+	{
+		return  0 ;
+	}
+
+	if( ( line = ml_edit_get_line( screen->edit , row)) == NULL)
+	{
+		return  0 ;
+	}
+
+	if( ( ch = ml_char_at( line , char_index)) == NULL)
+	{
+		return  0 ;
+	}
+	
+	if( ! ml_char_combine( ch , bytes , ch_size , cs , is_biwidth , is_comb ,
+		fg_color , bg_color , is_bold , is_underlined))
+	{
+		return  0 ;
+	}
+	
+	ml_line_set_modified( line , char_index , char_index) ;
+	
+	return  1 ;
+}
+
+int
+ml_screen_insert_chars(
+	ml_screen_t *  screen ,
+	ml_char_t *  chars ,
+	u_int  len
+	)
+{
+	return  ml_edit_insert_chars( screen->edit , chars , len) ;
+}
+
+int
+ml_screen_insert_new_lines(
+	ml_screen_t *  screen ,
+	u_int  size
+	)
+{
+	u_int  count ;
+
+	if( size > ml_edit_get_rows( screen->edit))
+	{
+		size = ml_edit_get_rows( screen->edit) ;
+	}
+
+	for( count = 0 ; count < size ; count ++)
+	{
+		ml_edit_insert_new_line( screen->edit) ;
+	}
+
+	return  1 ;
+}
+
+int
+ml_screen_overwrite_chars(
+	ml_screen_t *  screen ,
+	ml_char_t *  chars ,
+	u_int  len
+	)
+{
+	return  ml_edit_overwrite_chars( screen->edit , chars , len) ;
+}
+
+int
+ml_screen_delete_lines(
+	ml_screen_t *  screen ,
+	u_int  size
+	)
+{
+	u_int  count ;
+
+	if( size > ml_edit_get_rows( screen->edit))
+	{
+		size = ml_edit_get_rows( screen->edit) ;
+	}
+
+	for( count = 0 ; count < size ; count ++)
+	{
+		if( ! ml_edit_delete_line( screen->edit))
+		{
+		#ifdef  DEBUG
+			kik_warn_printf( KIK_DEBUG_TAG " deleting nonexisting lines.\n") ;
+		#endif
+			
+			return  0 ;
+		}
+	}
+	
+	return  1 ;
+}
+
+int
+ml_screen_go_forward(
+	ml_screen_t *  screen ,
+	u_int  size
+	)
+{
+	u_int  count ;
+
+	for( count = 0 ; count < size ; count ++)
+	{
+		if( ! ml_edit_go_forward( screen->edit , 0))
+		{
+		#ifdef  DEBUG
+			kik_warn_printf( KIK_DEBUG_TAG " cursor cannot go forward any more.\n") ;
+		#endif
+		
+			return  0 ;
+		}
+	}
+
+	return  1 ;
+}
+
+int
+ml_screen_go_back(
+	ml_screen_t *  screen ,
+	u_int  size
+	)
+{
+	u_int  count ;
+
+	for( count = 0 ; count < size ; count ++)
+	{
+		if( ! ml_edit_go_back( screen->edit , 0))
+		{
+		#ifdef  DEBUG
+			kik_warn_printf( KIK_DEBUG_TAG " cursor cannot go back any more.\n") ;
+		#endif
+					
+			return  0 ;
+		}
+	}
+
+	return  1 ;
+}
+
+int
+ml_screen_go_upward(
+	ml_screen_t *  screen ,
+	u_int  size
+	)
+{
+	u_int  count ;
+	
+	for( count = 0 ; count < size ; count ++)
+	{
+		if( ! ml_edit_go_upward( screen->edit , 0))
+		{
+		#ifdef  DEBUG
+			kik_warn_printf( KIK_DEBUG_TAG " cursor cannot go upward any more.\n") ;
+		#endif
+				
+			return  0 ;
+		}
+	}
+
+	return  1 ;
+}
+
+int
+ml_screen_go_downward(
+	ml_screen_t *  screen ,
+	u_int  size
+	)
+{
+	u_int  count ;
+	
+	for( count = 0 ; count < size ; count ++)
+	{
+		if( ! ml_edit_go_downward( screen->edit , 0))
+		{
+		#ifdef  DEBUG
+			kik_warn_printf( KIK_DEBUG_TAG " cursor cannot go downward any more.\n") ;
+		#endif
+		
+			return  0 ;
+		}
+	}
+
+	return  1 ;
+}
+
+int
+ml_screen_go_horizontally(
+	ml_screen_t *  screen ,
+	int  col
+	)
+{
+	return  ml_screen_goto( screen , col , ml_cursor_row( screen->edit)) ;
+}
+
+int
+ml_screen_go_vertically(
+	ml_screen_t *  screen ,
+	int  row
+	)
+{
+	return  ml_screen_goto( screen , ml_cursor_col( screen->edit) , row) ;
+}
+
+int
+ml_screen_cursor_visible(
+	ml_screen_t *  screen
+	)
+{
+	if( screen->is_cursor_visible)
+	{
+		return  1 ;
+	}
+	
+	screen->is_cursor_visible = 1 ;
+
+	return  1 ;
+}
+
+int
+ml_screen_cursor_invisible(
+	ml_screen_t *  screen
+	)
+{
+	if( ! screen->is_cursor_visible)
+	{
+		return  1 ;
+	}
+	
+	screen->is_cursor_visible = 0 ;
+
+	return  1 ;
+}
+
+/*
+ * XXX
+ * Note that alt_edit/normal_edit are directly switched by x_picture.c without
+ * using following 3 functions.
+ */
+
+int
+ml_screen_is_alternative_edit(
+	ml_screen_t *  screen
+        )
+{
+	if( screen->edit == &(screen->alt_edit) )
+	{
+		return  1 ;
+	}
+
+	return  0 ;
+}
+
+int
+ml_screen_use_normal_edit(
+	ml_screen_t *  screen
+	)
+{
+	screen->edit = &screen->normal_edit ;
+
+	if( screen->logvis)
+	{
+		(*screen->logvis->init)( screen->logvis ,
+			&screen->edit->model , &screen->edit->cursor) ;
+	}
+
+	ml_edit_set_modified_all( screen->edit) ;
+	
+	return  1 ;
+}
+
+int
+ml_screen_use_alternative_edit(
+	ml_screen_t *  screen
+	)
+{
+	screen->edit = &screen->alt_edit ;
+
+	ml_screen_goto_home( screen) ;
+	ml_screen_clear_below( screen) ;
+
+	if( screen->logvis)
+	{
+		(*screen->logvis->init)( screen->logvis ,
+			&screen->edit->model , &screen->edit->cursor) ;
+	}
+
+	ml_edit_set_modified_all( screen->edit) ;
+	
+	return  1 ;
+}
+
+int
+ml_screen_enable_local_echo(
+	ml_screen_t *  screen
+	)
+{
+	if( screen->stored_edits)
+	{
+		screen->stored_edits->time = clock() / (CLOCKS_PER_SEC/10) ;
+
+		return  1 ;
+	}
+
+	if( ! ( screen->stored_edits = malloc( sizeof( ml_stored_edits_t))))
+	{
+		return  0 ;
+	}
+
+	screen->stored_edits->normal_edit = screen->normal_edit ;
+	screen->stored_edits->alt_edit = screen->alt_edit ;
+
+	if( ! ml_edit_clone( &screen->normal_edit , &screen->stored_edits->normal_edit))
+	{
+		goto  error1 ;
+	}
+
+	if( ! ml_edit_clone( &screen->alt_edit , &screen->stored_edits->alt_edit))
+	{
+		goto  error2 ;
+	}
+
+	screen->stored_edits->time = clock() / (CLOCKS_PER_SEC/10) ;
+
+	return  1 ;
+
+error2:
+	ml_edit_final( &screen->stored_edits->normal_edit) ;
+
+error1:
+	free( screen->stored_edits) ;
+	screen->stored_edits = NULL ;
+
+	return  0 ;
+}
+
+int
+ml_screen_local_echo_wait(
+	ml_screen_t *  screen ,
+	int  msec		/* If 0 is specified, time is reset. */
+	)
+{
+	if( screen->stored_edits)
+	{
+		if( msec == 0)
+		{
+			screen->stored_edits->time = 0 ;
+		}
+		else if( screen->stored_edits->time + msec / 100 >= clock() / (CLOCKS_PER_SEC/10))
+		{
+			return  1 ;
+		}
+	}
+
+	return  0 ;
+}
+
+int
+ml_screen_disable_local_echo(
+	ml_screen_t *  screen
+	)
+{
+	u_int  row ;
+	u_int  num_of_rows ;
+	ml_line_t *  line ;
+
+	if( ! screen->stored_edits)
+	{
+		return  1 ;
+	}
+
+	num_of_rows = ml_edit_get_rows( screen->edit) ;
+
+	/* Modified lines are inherited to stored_edits. */
+	for( row = 0 ; row < num_of_rows ; row++)
+	{
+		if( ( line = ml_edit_get_line( screen->edit , row)) &&
+		    ml_line_is_modified( line) &&
+		    ( line = ml_edit_get_line(
+				( screen->edit == &screen->normal_edit ?
+					&screen->stored_edits->normal_edit :
+					&screen->stored_edits->alt_edit) , row)))
+		{
+			ml_line_set_modified_all( line) ;
+		}
+	}
+
+	ml_edit_final( &screen->normal_edit) ;
+	ml_edit_final( &screen->alt_edit) ;
+
+	screen->normal_edit = screen->stored_edits->normal_edit ;
+	screen->alt_edit = screen->stored_edits->alt_edit ;
+
+	free( screen->stored_edits) ;
+	screen->stored_edits = NULL ;
+
+	return  1 ;
+}
+
+int
+ml_screen_fill_all_with_e(
+	ml_screen_t *  screen
+	)
+{
+	ml_char_t  e_ch ;
+
+	ml_char_init( &e_ch) ;
+	ml_char_set( &e_ch , "E" , 1 , US_ASCII , 0 , 0 , ML_FG_COLOR , ML_BG_COLOR , 0 , 0) ;
+
+	ml_edit_fill_all( screen->edit , &e_ch) ;
+
+	ml_char_final( &e_ch) ;
+
+	return  1 ;
+}
