@@ -6,6 +6,7 @@
 #define  __ML_SCREEN_H__
 
 
+#include  <time.h>
 #include  <kiklib/kik_types.h>		/* int8_t */
 
 #include  "ml_edit.h"
@@ -26,14 +27,22 @@ typedef struct  ml_screen_event_listener
 
 typedef enum  ml_bs_mode
 {
-	BSM_NONE = 0x0,
-	BSM_VOLATILE,
-	BSM_STATIC,
+	BSM_NONE = 0x0 ,
+	BSM_DEFAULT ,
+	BSM_STATIC ,
 
 	BSM_MAX
 	
 } ml_bs_mode_t ;
 
+typedef struct  ml_stored_edits
+{
+	ml_edit_t  normal_edit ;
+	ml_edit_t  alt_edit ;
+	clock_t  time ;
+
+} ml_stored_edits_t ;
+
 typedef struct  ml_screen
 {
 	/* public(readonly) */
@@ -45,6 +54,7 @@ typedef struct  ml_screen
 
 	ml_edit_t  normal_edit ;
 	ml_edit_t  alt_edit ;
+	ml_stored_edits_t *  stored_edits ;	/* Store logical edits. */
 
 	ml_edit_scroll_event_listener_t  edit_scroll_listener ;
 	
@@ -140,9 +150,9 @@ int  ml_screen_visual( ml_screen_t *  screen) ;
 int  ml_screen_logical( ml_screen_t *  screen) ;
 
 #define  ml_screen_logical_visual_is_reversible( screen) \
-		((screen)->logvis && (screen)->logvis->is_reversible)
+		(! (screen)->logvis || (screen)->logvis->is_reversible)
 
-ml_bs_mode_t  ml_is_backscroll_mode( ml_screen_t *  screen) ;
+ml_bs_mode_t  ml_screen_is_backscrolling( ml_screen_t *  screen) ;
 
 int  ml_set_backscroll_mode( ml_screen_t *  screen , ml_bs_mode_t  mode) ;
 
@@ -179,6 +189,16 @@ int  ml_screen_get_line_region( ml_screen_t *  screen , int *  beg_row ,
 int  ml_screen_get_word_region( ml_screen_t *  screen , int *  beg_char_index ,
 	int *  beg_row , int *  end_char_index , int *  end_row , int  base_char_index , int  base_row) ;
 
+int  ml_screen_search_init( ml_screen_t *  screen ,
+	int (*match)( size_t * , size_t * , void * , u_char * , int)) ;
+
+int  ml_screen_search_final( ml_screen_t *  screen) ;
+
+int  ml_screen_search_reset_position( ml_screen_t *  screen) ;
+
+int  ml_screen_search_find( ml_screen_t *  screen , int *  beg_char_index , int *  beg_row ,
+	int *  end_char_index , int *  end_row , void *  regex , int  backward) ;
+
 
 /*
  * VT100 commands (called in logical context)
@@ -278,19 +298,21 @@ int  ml_screen_use_alternative_edit( ml_screen_t *  screen) ;
 
 int  ml_screen_is_alternative_edit( ml_screen_t *  screen) ;
 
+int  ml_screen_enable_local_echo( ml_screen_t *  screen) ;
+
+int  ml_screen_local_echo_wait( ml_screen_t *  screen , int  msec) ;
+
+int  ml_screen_disable_local_echo( ml_screen_t *  screen) ;
+
 int  ml_screen_fill_all_with_e( ml_screen_t *  screen) ;
 
-ml_bs_mode_t  ml_screen_is_backscrolling( ml_screen_t *  screen) ;
+#define  ml_screen_copy_area( screen , src_col , src_row , \
+		num_of_cols , num_of_rows , dst_col , dst_row) \
+		ml_edit_copy_area( (screen)->edit , src_col , src_row , \
+			num_of_cols , num_of_rows , dst_col , dst_row)
 
-int  ml_screen_search_init( ml_screen_t *  screen ,
-	int (*match)( size_t * , size_t * , void * , u_char * , int)) ;
-
-int  ml_screen_search_final( ml_screen_t *  screen) ;
-
-int  ml_screen_search_reset_position( ml_screen_t *  screen) ;
-
-int  ml_screen_search_find( ml_screen_t *  screen , int *  beg_char_index , int *  beg_row ,
-	int *  end_char_index , int *  end_row , void *  regex , int  backward) ;
+#define  ml_screen_erase_area( screen , col , row , num_of_cols , num_of_rows) \
+		ml_edit_erase_area( (screen)->edit , col , row , num_of_cols , num_of_rows)
 
 
 #endif
