@@ -13,6 +13,14 @@
 #include  "ml_screen.h"
 
 
+/* --- global variables --- */
+
+#ifndef  NO_IMAGE
+/* XXX */
+void (*ml_term_pty_closed_event)( ml_term_t *) ;
+#endif
+
+
 /* --- global functions --- */
 
 ml_term_t *
@@ -33,7 +41,10 @@ ml_term_new(
 	int  use_bce ,
 	int  use_dynamic_comb ,
 	ml_bs_mode_t  bs_mode ,
-	ml_vertical_mode_t  vertical_mode
+	ml_vertical_mode_t  vertical_mode ,
+	int  use_local_echo ,
+	char *  win_name ,
+	char *  icon_name
 	)
 {
 	ml_term_t *  term ;
@@ -57,8 +68,8 @@ ml_term_new(
 		goto  error ;
 	}
 
-	if( ( term->parser = ml_vt100_parser_new( term->screen , encoding , policy , col_size_a ,
-				use_char_combining , use_multi_col_char)) == NULL)
+	if( ! ( term->parser = ml_vt100_parser_new( term->screen , encoding , policy , col_size_a ,
+				use_char_combining , use_multi_col_char , win_name , icon_name)))
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " ml_vt100_parser_new failed.\n") ;
@@ -84,6 +95,8 @@ ml_term_new(
 
 	term->is_auto_encoding = is_auto_encoding ;
 
+	term->use_local_echo = use_local_echo ;
+
 	return  term ;
 
 error:
@@ -107,6 +120,13 @@ ml_term_delete(
 	ml_term_t *  term
 	)
 {
+#ifndef  NO_IMAGE
+	if( ml_term_pty_closed_event)
+	{
+		(*ml_term_pty_closed_event)( term) ;
+	}
+#endif
+
 	if( term->pty)
 	{
 		ml_pty_delete( term->pty) ;
@@ -117,8 +137,6 @@ ml_term_delete(
 		(*term->shape->delete)( term->shape) ;
 	}
 
-	free( term->win_name) ;
-	free( term->icon_name) ;
 	free( term->icon_path) ;
 
 	ml_screen_delete( term->screen) ;
@@ -340,6 +358,22 @@ ml_term_set_use_dynamic_comb(
 	return  1 ;
 }
 
+int
+ml_term_set_use_local_echo(
+	ml_term_t *  term ,
+	int  flag
+	)
+{
+	if( term->use_local_echo != flag && ! ( term->use_local_echo = flag))
+	{
+		ml_screen_logical( term->screen) ;
+		ml_screen_disable_local_echo( term->screen) ;
+		ml_screen_visual( term->screen) ;
+	}
+
+	return  1 ;
+}
+
 int
 ml_term_get_master_fd(
 	ml_term_t *  term
@@ -415,6 +449,11 @@ ml_term_write(
 			return  0 ;
 		}
 
+		if( term->use_local_echo)
+		{
+			ml_vt100_parser_local_echo( term->parser , buf , len) ;
+		}
+
 		return  ml_write_to_pty( term->pty , buf , len) ;
 	}
 }
@@ -513,6 +552,15 @@ ml_term_set_modified_region_in_screen(
 	int  row ;
 	ml_line_t *  line ;
 
+#ifdef  DEBUG
+	if( term->screen->logvis && ! term->screen->logvis->is_visual)
+	{
+		kik_debug_printf( KIK_DEBUG_TAG
+			" ml_term_set_modified_lines() should be called in visual context but"
+			" is called in logical context.\n") ;
+	}
+#endif
+
 	if( ! ml_screen_logical_visual_is_reversible( term->screen))
 	{
 		ml_screen_logical( term->screen) ;
@@ -873,30 +921,6 @@ ml_term_enter_backscroll_mode(
 	return  ml_enter_backscroll_mode( term->screen) ;
 }
 
-int
-ml_term_set_window_name(
-	ml_term_t *  term ,
-	char *  name
-	)
-{
-	free( term->win_name) ;
-	term->win_name = strdup( name) ;
-
-	return  1 ;
-}
-
-int
-ml_term_set_icon_name(
-	ml_term_t *  term ,
-	char *  name
-	)
-{
-	free( term->icon_name) ;
-	term->icon_name = strdup( name) ;
-
-	return  1 ;
-}
-
 int
 ml_term_set_icon_path(
 	ml_term_t *  term ,
