@@ -41,8 +41,6 @@ typedef struct ml_term
 	ml_vertical_mode_t  vertical_mode ;
 	ml_bidi_mode_t  bidi_mode ;
 
-	char *  win_name ;
-	char *  icon_name ;
 	char *  icon_path ;
 
 	int8_t  use_bidi ;
@@ -51,16 +49,22 @@ typedef struct ml_term
 
 	int8_t  is_auto_encoding ;
 
+	int8_t  use_local_echo ;
+
 	int8_t  is_attached ;
 
 } ml_term_t ;
 
 
+/* XXX */
+extern void (*ml_term_pty_closed_event)( ml_term_t *) ;
+
 ml_term_t *  ml_term_new( u_int  cols , u_int  rows , u_int  tab_size , u_int  log_size ,
 	ml_char_encoding_t  encoding , int  is_auto_encoding , ml_unicode_policy_t  policy ,
 	u_int  col_size_a , int  use_char_combining , int  use_multi_col_char , int  use_bidi ,
 	ml_bidi_mode_t  bidi_mode , int  use_ind , int  use_bce , int  use_dynamic_comb ,
-	ml_bs_mode_t  bs_mode , ml_vertical_mode_t  vertical_mode) ;
+	ml_bs_mode_t  bs_mode , ml_vertical_mode_t  vertical_mode , int  use_local_echo ,
+	char *  win_name , char *  icon_name) ;
 
 int  ml_term_delete( ml_term_t *  term) ;
 
@@ -91,9 +95,6 @@ int  ml_term_set_auto_encoding( ml_term_t *  term , int  is_auto_encoding) ;
 
 #define  ml_term_is_auto_encoding( term)  ((term)->is_auto_encoding)
 
-#define  ml_term_set_unicode_policy( term , policy) \
-		ml_vt100_parser_set_unicode_policy( (term)->parser , policy)
-
 int  ml_term_set_use_bidi( ml_term_t *  term , int  flag) ;
 
 #define  ml_term_is_using_bidi( term)  ((term)->use_bidi)
@@ -114,6 +115,11 @@ int  ml_term_set_use_dynamic_comb( ml_term_t *  term , int  flag) ;
 
 #define  ml_term_is_using_dynamic_comb( term)  ((term)->use_dynamic_comb)
 
+/* Must be called in visual context. */
+int  ml_term_set_use_local_echo( ml_term_t *  term , int  flag) ;
+
+#define  ml_term_is_using_local_echo( term)  ((term)->use_local_echo)
+
 #define  ml_term_convert_to( term , dst , len , _parser) \
 		ml_vt100_parser_convert_to( (term)->parser , dst , len , _parser)
 
@@ -133,7 +139,7 @@ size_t  ml_term_write( ml_term_t *  term , u_char *  buf , size_t  len , int  to
 
 /* Must be called in visual context. */
 #define  ml_term_write_loopback( term , buf , len) \
-		ml_parse_vt100_write_loopback( (term)->parser , buf , len)
+		ml_vt100_parser_write_loopback( (term)->parser , buf , len)
 
 int  ml_term_resize( ml_term_t *  term , u_int  cols , u_int  rows) ;
 
@@ -280,21 +286,24 @@ int  ml_term_enter_backscroll_mode( ml_term_t *  term) ;
 
 #define  ml_term_want_focus_event( term)  ml_vt100_parser_want_focus_event((term)->parser)
 
-int  ml_term_set_window_name( ml_term_t *  term , char *  name) ;
-
-int  ml_term_set_icon_name( ml_term_t *  term , char *  name) ;
+#define  ml_term_modify_other_keys( term)  ml_vt100_parser_modify_other_keys((term)->parser)
 
 int  ml_term_set_icon_path( ml_term_t *  term , char *  path) ;
 
-#define  ml_term_window_name( term)  ((term)->win_name)
+#define  ml_term_window_name( term)  ml_get_window_name((term)->parser)
 
-#define  ml_term_icon_name( term)  ((term)->icon_name)
+#define  ml_term_icon_name( term)  ml_get_icon_name((term)->parser)
 
 #define  ml_term_icon_path( term)  ((term)->icon_path)
 
 #define  ml_term_is_bracketed_paste_mode( term) \
 		ml_vt100_parser_is_bracketed_paste_mode((term)->parser)
 
+#define  ml_term_set_unicode_policy( term , policy) \
+		ml_vt100_parser_set_unicode_policy((term)->parser , policy)
+
+#define  ml_term_get_unicode_policy( term)  ml_vt100_parser_get_unicode_policy((term)->parser)
+
 int  ml_term_start_config_menu( ml_term_t *  term , char *  cmd_path ,
 		int  x , int  y , char *  display) ;
 
