@@ -31,7 +31,9 @@ typedef enum  ml_unicode_policy
 	NOT_USE_UNICODE_FONT = 0x1 ,
 	USE_UNICODE_PROPERTY = 0x2 ,
 	ONLY_USE_UNICODE_FONT = 0x4 ,
-	
+	NOT_USE_UNICODE_BOXDRAW_FONT = 0x8 ,
+	ONLY_USE_UNICODE_BOXDRAW_FONT = 0x10 ,
+
 	UNICODE_POLICY_MAX
 
 } ml_unicode_policy_t ;
@@ -93,6 +95,10 @@ typedef struct  ml_xterm_event_listener
 	int (*im_is_active)( void *) ;			/* called in logical context. */
 	void (*switch_im_mode)( void *) ;		/* called in logical context. */
 	void (*set_selection)( void * , ml_char_t * , u_int) ;	/* called in logical context. */
+	int (*get_window_size)( void * , u_int * , u_int *) ;	/* called in logical context. */
+	int (*get_rgb)( void * , u_int8_t * , u_int8_t * ,
+			u_int8_t * , ml_color_t) ;		/* called in logical context. */
+	ml_char_t *  (*get_picture_data)( void * , char * , int * , int *) ; /* called in logical context. */
 
 } ml_xterm_event_listener_t ;
 
@@ -131,6 +137,12 @@ typedef struct  ml_vt100_storable_states
 
 } ml_vt100_storable_states_t ;
 
+typedef struct ml_vt100_saved_names
+{
+	char **  names ;
+	u_int  num ;
+
+} ml_vt100_saved_names_t ;
 
 typedef struct  ml_vt100_parser
 {
@@ -155,16 +167,19 @@ typedef struct  ml_vt100_parser
 
 	int  log_file ;
 
+	char *  win_name ;
+	char *  icon_name ;
+
 	/* ml_unicode_policy_t */ int8_t  unicode_policy ;
 
 	/* ml_mouse_report_mode_t */ int8_t  mouse_mode ;
 	/* ml_extended_mouse_report_mode_t */ int8_t  ext_mouse_mode ;
 
 	/* Used for non iso2022 encoding */
-	int8_t  is_dec_special_in_gl ;
+	mkf_charset_t  gl ;
+	mkf_charset_t  g0 ;
+	mkf_charset_t  g1 ;
 	int8_t  is_so ;
-	int8_t  is_dec_special_in_g0 ;
-	int8_t  is_dec_special_in_g1 ;
 
 	int8_t  is_bold ;
 	int8_t  is_underlined ;
@@ -185,6 +200,12 @@ typedef struct  ml_vt100_parser
 
 	int8_t  im_is_active ;
 
+#if  0
+	int8_t  modify_cursor_keys ;
+	int8_t  modify_function_keys ;
+#endif
+	int8_t  modify_other_keys ;
+
 #ifdef  USE_VT52
 	int8_t  is_vt52_mode ;
 #endif
@@ -193,12 +214,20 @@ typedef struct  ml_vt100_parser
 	ml_vt100_storable_states_t  saved_normal ;
 	ml_vt100_storable_states_t  saved_alternate ;
 
+	ml_vt100_saved_names_t  saved_win_names ;
+	ml_vt100_saved_names_t  saved_icon_names ;
+
 } ml_vt100_parser_t ;
 
 
+void  ml_set_use_alt_buffer( int  use) ;
+
+void  ml_set_use_ansi_colors( int  use) ;
+
 ml_vt100_parser_t *  ml_vt100_parser_new( ml_screen_t *  screen , ml_char_encoding_t  encoding ,
 	ml_unicode_policy_t  policy , u_int  col_size_a ,
-	int  use_char_combining , int  use_multi_col_char) ;
+	int  use_char_combining , int  use_multi_col_char ,
+	char *  win_name , char *  icon_name) ;
 
 int  ml_vt100_parser_delete( ml_vt100_parser_t *  vt100_parser) ;
 
@@ -210,14 +239,15 @@ int  ml_vt100_parser_set_xterm_listener( ml_vt100_parser_t *  vt100_parser ,
 int  ml_vt100_parser_set_config_listener( ml_vt100_parser_t *  vt100_parser ,
 	ml_config_event_listener_t *  config_listener) ;
 
-int  ml_vt100_parser_set_unicode_policy( ml_vt100_parser_t *  vt100_parser ,
-	ml_unicode_policy_t  policy) ;
-
 int  ml_parse_vt100_sequence( ml_vt100_parser_t *  vt100_parser) ;
 
-int  ml_parse_vt100_write_loopback( ml_vt100_parser_t *  vt100_parser ,
+/* Must be called in visual context. */
+int  ml_vt100_parser_write_loopback( ml_vt100_parser_t *  vt100_parser ,
 	u_char *  buf , size_t  len) ;
 
+/* Must be called in visual context. */
+int  ml_vt100_parser_local_echo( ml_vt100_parser_t *  vt100_parser , u_char *  buf , size_t  len) ;
+
 int  ml_vt100_parser_change_encoding( ml_vt100_parser_t *  vt100_parser ,
 	ml_char_encoding_t  encoding) ;
 
@@ -230,6 +260,10 @@ int  ml_init_encoding_parser( ml_vt100_parser_t *  vt100_parser) ;
 
 int  ml_init_encoding_conv( ml_vt100_parser_t *  vt100_parser) ;
 
+#define  ml_get_window_name( vt100_parser)  ((vt100_parser)->win_name)
+
+#define  ml_get_icon_name( vt100_parser)  ((vt100_parser)->icon_name)
+
 int  ml_vt100_parser_set_col_size_of_width_a( ml_vt100_parser_t *  vt100_parser ,
 	u_int  col_size_a) ;
 
@@ -266,5 +300,15 @@ int  ml_vt100_parser_set_logging_vt_seq( ml_vt100_parser_t *  vt100_parser , int
 
 #define  ml_vt100_parser_want_focus_event( vt100_parser)  ((vt100_parser)->want_focus_event)
 
+#define  ml_vt100_parser_modify_other_keys( vt100_parser)  ((vt100_parser)->modify_other_keys)
+
+int  ml_vt100_parser_set_unicode_policy( ml_vt100_parser_t *  vt100_parser ,
+		ml_unicode_policy_t  policy) ;
+
+#define  ml_vt100_parser_get_unicode_policy( vt100_parser)  ((vt100_parser)->unicode_policy)
+
+int  ml_convert_to_internal_ch( mkf_char_t *  ch , ml_unicode_policy_t  unicode_policy ,
+		mkf_charset_t  gl) ;
+
 
 #endif
