@@ -4,6 +4,7 @@
 
 #include  "../x_font.h"
 
+#include  <stdlib.h>	/* strtod */
 #ifdef  USE_TYPE_XFT
 #include  <X11/Xft/Xft.h>
 #endif
@@ -200,7 +201,6 @@ parse_fc_font_name(
 					{ "semi-condensed" , /* XXX This style is ignored. */
 						0 , 0 , } ,
 				} ;
-				float  size_f ;
 				
 				for( count = 0 ; count < sizeof(styles) / sizeof(styles[0]) ;
 					count ++)
@@ -227,21 +227,25 @@ parse_fc_font_name(
 						goto  next_char ;
 					}
 				}
-				
-				if( *p != '0' &&	/* In case of "DevLys 010" font family. */
-				    sscanf( p , "%f" , &size_f) == 1)
-				{
-					/* If matched with %f, p has no more parameters. */
 
-					*orig_p = '\0' ;
-					*font_size = size_f ;
-
-					break ;
-				}
-				else
+				if( *p != '0') /* In case of "DevLys 010" font family. */
 				{
-					step = 1 ;
+					char *  end ;
+					double  size ;
+
+					size = strtod( p , &end) ;
+					if( *end == '\0')
+					{
+						/* p has no more parameters. */
+
+						*orig_p = '\0' ;
+						*font_size = size ;
+
+						break ;
+					}
 				}
+
+				step = 1 ;
 			}
 		}
 		else
@@ -603,6 +607,12 @@ fc_set_font(
 
 	slant = -1 ;	/* use default value */
 
+	/*
+	 * x_off related to percent is set before ft_font_open while
+	 * x_off related to is_vertical and letter_space is set after.
+	 */
+	font->x_off = 0 ;
+
 	if( fontname)
 	{
 		char *  p ;
@@ -628,9 +638,24 @@ fc_set_font(
 			{
 				/* basic font (e.g. usascii) width */
 
+				/* if font->is_var_col_width is true, 0 is returned. */
 				ch_width = get_fc_col_width( font , fontsize_d , percent ,
 								letter_space) ;
 
+				if( percent > 100 && use_xft)  /* cairo ignores percent. */
+				{
+					/*
+					 * Centering
+					 * (fontsize * percent / 100 + letter_space = ch_width
+					 *  -> fontsize = (ch_width - letter_space) * 100 / percent
+					 *  -> fontsize * (percent - 100) / 100
+					 *     = (ch_width - letter_space) * (percent - 100)
+					 *       / percent)
+					 */
+					font->x_off = (ch_width - letter_space) *
+							(percent - 100) / percent / 2 ;
+				}
+
 				if( font->is_vertical)
 				{
 					/*
@@ -674,6 +699,8 @@ fc_set_font(
 			{
 				goto  font_found ;
 			}
+
+			font->x_off = 0 ;
 		}
 
 		kik_msg_printf( "Font %s (for size %f) couldn't be loaded.\n" ,
@@ -729,7 +756,6 @@ fc_set_font(
 
 font_found:
 
-	font->x_off = 0 ;
 	font->is_proportional = font->is_var_col_width ;
 
 	if( use_xft)
@@ -767,7 +793,17 @@ font_found:
 		else
 		{
 			font->width = ch_width ;
+
+			if( font->is_vertical && font->cols == 1)
+			{
+				font->x_off += ch_width / 4 ;	/* Centering */
+			}
+			else
+			{
+				font->x_off += letter_space * font->cols / 2 ;
+			}
 		}
+
 	#endif	/* USE_TYPE_XFT */
 	}
 	else
@@ -792,7 +828,12 @@ font_found:
 		font->height = DOUBLE_ROUNDUP_TO_INT(extents.height) ;
 		font->ascent = DOUBLE_ROUNDUP_TO_INT(extents.ascent) ;
 
-		/* XXX letter_space is always ignored. */
+		/*
+		 * Set letter_space here because cairo_font_open() ignores it.
+		 * (FC_CHAR_WIDTH doesn't make effect in cairo.)
+		 * Then, column width for vertical mode should be set again here.
+		 */
+
 		if( font->cols == 2)
 		{
 			font->width = DOUBLE_ROUNDUP_TO_INT(extents.max_x_advance) ;
@@ -800,6 +841,21 @@ font_found:
 		else
 		{
 			font->width = cairo_calculate_char_width( font , "W" , 1) ;
+
+			if( font->is_vertical)
+			{
+				font->is_proportional = 1 ;
+				font->width *= 2 ;
+				font->x_off = font->width / 4 ;	/* Centering */
+			}
+		}
+
+		/* letter_space is ignored in variable column width mode. */
+		if( ! font->is_var_col_width && letter_space > 0)
+		{
+			font->is_proportional = 1 ;
+			font->width += (letter_space * font->cols) ;
+			font->x_off += (letter_space * font->cols / 2) ;  /* Centering */
 		}
 
 		if( col_width > 0 /* is not usascii */ && ! font->is_proportional &&
