@@ -10,7 +10,6 @@
 #include  <kiklib/kik_str.h>	/* kik_snprintf */
 #include  <kiklib/kik_mem.h>	/* alloca */
 #include  <kiklib/kik_str.h>	/* kik_str_sep/kik_str_to_int */
-#include  <kiklib/kik_util.h>	/* DIGIT_STR_LEN/K_MIN */
 #include  <kiklib/kik_locale.h>	/* kik_get_lang() */
 #include  <mkf/mkf_ucs4_map.h>
 #include  <ml_char_encoding.h>	/* x_convert_to_xft_ucs4 */
@@ -285,8 +284,7 @@ parse_font_name(
 					{ "semi-condensed" , /* XXX This style is ignored. */
 						0 , 0 , } ,
 				} ;
-				float  size_f ;
-				
+
 				for( count = 0 ; count < sizeof(styles) / sizeof(styles[0]) ;
 					count ++)
 				{
@@ -312,20 +310,25 @@ parse_font_name(
 						goto  next_char ;
 					}
 				}
-				
-				if( sscanf( p , "%f" , &size_f) == 1)
-				{
-					/* If matched with %f, p has no more parameters. */
 
-					*orig_p = '\0' ;
-					*font_size = size_f ;
-
-					break ;
-				}
-				else
+				if( *p != '0') /* In case of "DevLys 010" font family. */
 				{
-					step = 1 ;
+					char *  end ;
+					double  size ;
+
+					size = strtod( p , &end) ;
+					if( *end == '\0')
+					{
+						/* p has no more parameters. */
+
+						*orig_p = '\0' ;
+						*font_size = size ;
+
+						break ;
+					}
 				}
+
+				step = 1 ;
 			}
 		}
 		else
@@ -373,7 +376,7 @@ x_font_new(
 	double  fontsize_d ;
 	u_int  percent ;
 	
-	if( ( font = malloc( sizeof( x_font_t))) == NULL)
+	if( type_engine != TYPE_XCORE || ( font = malloc( sizeof( x_font_t))) == NULL)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " malloc() failed.\n") ;
@@ -589,7 +592,7 @@ x_font_new(
 				ch_width = fontsize * percent / 200 ;
 			}
 
-			if( font->width != ch_width)
+			if( ! font->is_var_col_width && font->width != ch_width)
 			{
 				font->is_proportional = 1 ;
 
@@ -641,7 +644,8 @@ x_font_new(
 			{
 				font->is_proportional = 1 ;
 
-				if( font->width < col_width)
+				/* is_var_col_width is always false if is_vertical is true. */
+				if( /* ! font->is_var_col_width && */ font->width < col_width)
 				{
 					font->x_off = (col_width - font->width) / 2 ;
 				}
@@ -659,7 +663,8 @@ x_font_new(
 
 				font->is_proportional = 1 ;
 
-				if( font->width < col_width * font->cols)
+				if( ! font->is_var_col_width &&
+				    font->width < col_width * font->cols)
 				{
 					font->x_off = (col_width * font->cols - font->width) / 2 ;
 				}
@@ -755,86 +760,101 @@ x_calculate_char_width(
 	x_font_t *  font ,
 	const u_char *  ch ,
 	size_t  len ,
-	mkf_charset_t  cs
+	mkf_charset_t  cs ,
+	int *  draw_alone
 	)
 {
-	if( font->is_var_col_width && font->is_proportional)
+	if( draw_alone)
 	{
-		SIZE  sz ;
+		*draw_alone = 0 ;
+	}
 
-		if( ! display_gc)
+	if( font->is_proportional)
+	{
+		if( font->is_var_col_width)
 		{
-			/*
-			 * Cached as far as x_caculate_char_width is called.
-			 * display_gc is deleted in x_font_new or x_font_delete.
-			 */
-			display_gc = CreateIC( "Display" , NULL , NULL , NULL) ;
-		}
+			SIZE  sz ;
 
-		SelectObject( display_gc , font->fid) ;
-
-		if( cs != US_ASCII && ! IS_ISCII(cs))
-		{
-			u_char  ucs4_bytes[4] ;
-			u_char  utf16_bytes[4] ;
-
-			if( cs == ISO10646_UCS4_1)
+			if( ! display_gc)
 			{
-				memcpy( ucs4_bytes , ch , 4) ;
+				/*
+				 * Cached as far as x_caculate_char_width is called.
+				 * display_gc is deleted in x_font_new or x_font_delete.
+				 */
+				display_gc = CreateIC( "Display" , NULL , NULL , NULL) ;
+			}
+
+			SelectObject( display_gc , font->fid) ;
+
+			if( cs != US_ASCII && ! IS_ISCII(cs))
+			{
+				u_char  ucs4_bytes[4] ;
+				u_char  utf16_bytes[4] ;
+
+				if( cs == ISO10646_UCS4_1)
+				{
+					memcpy( ucs4_bytes , ch , 4) ;
+				}
+				else
+				{
+					mkf_char_t  non_ucs ;
+					mkf_char_t  ucs4 ;
+
+					if( ml_is_msb_set( cs))
+					{
+						size_t  count ;
+
+						for( count = 0 ; count < len ; count ++)
+						{
+							non_ucs.ch[count] = ch[count] & 0x7f ;
+						}
+					}
+					else
+					{
+						memcpy( non_ucs.ch , ch , len) ;
+					}
+
+					non_ucs.size = len ;
+					non_ucs.property = 0 ;
+					non_ucs.cs = cs ;
+
+					if( mkf_map_to_ucs4( &ucs4 , &non_ucs))
+					{
+						memcpy( ucs4_bytes , ucs4.ch , 4) ;
+					}
+					else
+					{
+						goto  fixed_col_width ;
+					}
+				}
+
+				if( ! ( len = x_convert_ucs4_to_utf16( utf16_bytes ,
+						ucs4_bytes)) ||
+				    ! GetTextExtentPointW( display_gc ,
+						utf16_bytes , len / 2 , &sz))
+				{
+					goto  fixed_col_width ;
+				}
 			}
 			else
 			{
-				mkf_char_t  non_ucs ;
-				mkf_char_t  ucs4 ;
-
-				if( ml_is_msb_set( cs))
+				if( ! GetTextExtentPointA( display_gc , ch , len , &sz))
 				{
-					size_t  count ;
-
-					for( count = 0 ; count < len ; count ++)
-					{
-						non_ucs.ch[count] = ch[count] & 0x7f ;
-					}
-				}
-				else
-				{
-					memcpy( non_ucs.ch , ch , len) ;
-				}
-
-				non_ucs.size = len ;
-				non_ucs.property = 0 ;
-				non_ucs.cs = cs ;
-
-				if( mkf_map_to_ucs4( &ucs4 , &non_ucs))
-				{
-					memcpy( ucs4_bytes , ucs4.ch , 4) ;
-				}
-				else
-				{
-					return  0 ;
+					goto  fixed_col_width ;
 				}
 			}
 
-			if( ! ( len = x_convert_ucs4_to_utf16( utf16_bytes , ucs4_bytes)) ||
-			    ! GetTextExtentPointW( display_gc , utf16_bytes , len / 2 , &sz))
-			{
-				return  0 ;
-			}
+			return  sz.cx ;
 		}
-		else
+
+	fixed_col_width:
+		if( draw_alone)
 		{
-			if( ! GetTextExtentPointA( display_gc , ch , len , &sz))
-			{
-				return  0 ;
-			}
+			*draw_alone = 1 ;
 		}
+	}
 
-		return  sz.cx ;
-	}
-	else
-	{
-		return  font->width ;
-	}
+	return  font->width ;
 }
 
 char **
