@@ -59,10 +59,45 @@ static int  click_interval = 250 ;	/* millisecond, same as xterm. */
 static mkf_parser_t *  m_cp_parser ;
 static LONG  decorate_width ;
 static LONG  decorate_height ;		/* Height of Title bar etc. */
+static int  use_urgent_bell ;
 
 
 /* --- static functions --- */
 
+#ifdef  FLASHW_ALL
+static void
+urgent_bell(
+	x_window_t *  win ,
+	int  on
+	)
+{
+	if( use_urgent_bell && ( ! win->is_focused || ! on))
+	{
+		win = x_get_root_window( win) ;
+
+		if( on)
+		{
+			FLASHWINFO  info ;
+
+			info.cbSize = sizeof(info) ;
+			info.hwnd = win->my_window ;
+			info.dwFlags = FLASHW_ALL ;
+			info.uCount = 5 ;
+			info.dwTimeout = 0 ;	/* standard */
+
+			FlashWindowEx( &info) ;
+		}
+		else
+		{
+			FlashWindow( win->my_window , FALSE) ;
+		}
+	}
+}
+#else
+#define  urgent_bell(win,on)  (0)
+#endif
+
+
 static int
 set_transparent(
 	x_window_t *  win ,
@@ -690,6 +725,102 @@ update_decorate_size(
 	}
 }
 
+static int
+clear_margin_area(
+	x_window_t *  win
+	)
+{
+	if( win->margin == 0)
+	{
+		return  1 ;
+	}
+	else if( win->gc->gc == None)
+	{
+		return  0 ;
+	}
+	else
+	{
+		if( win->wall_picture)
+		{
+			BitBlt( win->gc->gc , 0, 0, win->margin, ACTUAL_HEIGHT(win),
+				win->wall_picture , 0 , 0 , SRCCOPY) ;
+			BitBlt( win->gc->gc, win->margin, 0,
+				win->width + win->margin, win->margin,
+				win->wall_picture , win->margin , 0 , SRCCOPY) ;
+			BitBlt( win->gc->gc, win->width + win->margin, 0,
+				ACTUAL_WIDTH(win), ACTUAL_HEIGHT(win) ,
+				win->wall_picture , win->width + win->margin, 0 , SRCCOPY) ;
+			BitBlt( win->gc->gc, win->margin, win->height + win->margin,
+				win->width + win->margin, ACTUAL_HEIGHT(win) ,
+				win->wall_picture, win->margin, win->height + win->margin,
+				SRCCOPY) ;
+		}
+		else
+		{
+			HBRUSH  brush ;
+			RECT  r ;
+
+			brush = x_acquire_brush( win->bg_color.pixel) ;
+
+			SetRect( &r, 0, 0, win->margin, ACTUAL_HEIGHT(win)) ;
+			FillRect( win->gc->gc , &r , brush) ;
+			SetRect( &r, win->margin, 0, win->width + win->margin, win->margin) ;
+			FillRect( win->gc->gc , &r , brush) ;
+			SetRect( &r, win->width + win->margin, 0,
+				ACTUAL_WIDTH(win), ACTUAL_HEIGHT(win)) ;
+			FillRect( win->gc->gc , &r , brush) ;
+			SetRect( &r, win->margin, win->height + win->margin,
+				win->width + win->margin, ACTUAL_HEIGHT(win)) ;
+			FillRect( win->gc->gc , &r , brush) ;
+
+			x_release_brush( brush) ;
+		}
+		
+		return  1 ;
+	}
+}
+
+static WORD
+oem_key_to_char(
+	WORD  wparam
+	)
+{
+	if( VK_OEM_1 <= wparam && wparam <= VK_OEM_2)
+	{
+		/*
+		 * VK_OEM_1 0xba
+		 * VK_OEM_PLUS 0xbb
+		 * VK_OEM_COMMA 0xbc
+		 * VK_OEM_MINUS 0xbd
+		 * VK_OEM_PERIOD 0xbe
+		 * VK_OEM_2 0xbf
+		 */
+		return  '*' + wparam - VK_OEM_1 ;
+	}
+	else if( wparam == VK_OEM_3)
+	{
+		return  '@' ;
+	}
+	else if( VK_OEM_4 <= wparam && wparam <= VK_OEM_7)
+	{
+		/*
+		 * VK_OEM_4 0xdb
+		 * VK_OEM_5 0xdc
+		 * VK_OEM_6 0xdd
+		 * VK_OEM_7 0xde
+		 */
+		return  '[' + wparam - VK_OEM_4 ;
+	}
+	else if( wparam == VK_OEM_102)
+	{
+		return  '_' ;
+	}
+	else
+	{
+		return  0 ;
+	}
+}
+
 
 /* --- global functions --- */
 
@@ -1043,7 +1174,7 @@ x_window_show(
 			   specify XValue|YValue to localte window at win->x/win->y. */
 	)
 {
-	int  count ;
+	u_int  count ;
 
 	if( win->my_window)
 	{
@@ -1099,6 +1230,10 @@ x_window_show(
 		return  0 ;
 	}
 
+#ifndef  DISABLE_XDND
+	DragAcceptFiles( win->my_window , TRUE) ;
+#endif
+
 	/*
 	 * This should be called after Window Manager settings, because
 	 * x_set_{window|icon}_name() can be called in win->window_realized().
@@ -1417,61 +1552,6 @@ x_window_clear(
 	}
 }
 
-int
-x_window_clear_margin_area(
-	x_window_t *  win
-	)
-{
-	if( win->margin == 0)
-	{
-		return  1 ;
-	}
-	else if( win->gc->gc == None)
-	{
-		return  0 ;
-	}
-	else
-	{
-		if( win->wall_picture)
-		{
-			BitBlt( win->gc->gc , 0, 0, win->margin, ACTUAL_HEIGHT(win),
-				win->wall_picture , 0 , 0 , SRCCOPY) ;
-			BitBlt( win->gc->gc, win->margin, 0,
-				win->width + win->margin, win->margin,
-				win->wall_picture , win->margin , 0 , SRCCOPY) ;
-			BitBlt( win->gc->gc, win->width + win->margin, 0,
-				ACTUAL_WIDTH(win), ACTUAL_HEIGHT(win) ,
-				win->wall_picture , win->width + win->margin, 0 , SRCCOPY) ;
-			BitBlt( win->gc->gc, win->margin, win->height + win->margin,
-				win->width + win->margin, ACTUAL_HEIGHT(win) ,
-				win->wall_picture, win->margin, win->height + win->margin,
-				SRCCOPY) ;
-		}
-		else
-		{
-			HBRUSH  brush ;
-			RECT  r ;
-
-			brush = x_acquire_brush( win->bg_color.pixel) ;
-
-			SetRect( &r, 0, 0, win->margin, ACTUAL_HEIGHT(win)) ;
-			FillRect( win->gc->gc , &r , brush) ;
-			SetRect( &r, win->margin, 0, win->width + win->margin, win->margin) ;
-			FillRect( win->gc->gc , &r , brush) ;
-			SetRect( &r, win->width + win->margin, 0,
-				ACTUAL_WIDTH(win), ACTUAL_HEIGHT(win)) ;
-			FillRect( win->gc->gc , &r , brush) ;
-			SetRect( &r, win->margin, win->height + win->margin,
-				win->width + win->margin, ACTUAL_HEIGHT(win)) ;
-			FillRect( win->gc->gc , &r , brush) ;
-
-			x_release_brush( brush) ;
-		}
-		
-		return  1 ;
-	}
-}
-
 int
 x_window_clear_all(
 	x_window_t *  win
@@ -1604,12 +1684,36 @@ x_window_update(
 	return  1 ;
 }
 
+int
+x_window_update_all(
+	x_window_t *  win
+	)
+{
+	u_int  count ;
+
+	clear_margin_area( win) ;
+
+#if  0
+	if( win->window_exposed)
+	{
+		(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+	}
+#endif
+
+	for( count = 0 ; count < win->num_of_children ; count ++)
+	{
+		x_window_update_all( win->children[count]) ;
+	}
+
+	return  1 ;
+}
+
 void
 x_window_idling(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
@@ -1644,7 +1748,7 @@ x_window_receive_event(
 	XEvent *  event
 	)
 {
-	int  count ;
+	u_int  count ;
 
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
@@ -1760,7 +1864,7 @@ x_window_receive_event(
 
 			if( margin_area_exposed)
 			{
-				x_window_clear_margin_area( win) ;
+				clear_margin_area( win) ;
 			}
 
 			(*win->window_exposed)( win, x, y, width, height) ;
@@ -1799,12 +1903,13 @@ x_window_receive_event(
 		{
 			XKeyEvent  kev ;
 
+			kev.state = get_key_state() ;
+
 			if( ( 0x30 <= event->wparam && event->wparam <= VK_DIVIDE) ||
 				event->wparam == VK_BACK || event->wparam == VK_TAB ||
 				event->wparam == VK_RETURN || event->wparam == VK_ESCAPE ||
 				event->wparam == VK_SPACE)
 			{
-				kev.state = get_key_state() ;
 				kev.ch = event->wparam ;
 
 				if( ( kev.state & ModMask) && event->msg == WM_SYSKEYDOWN)
@@ -1837,38 +1942,51 @@ x_window_receive_event(
 			}
 			else if( event->msg == WM_SYSKEYDOWN)
 			{
-				break ;
+				if( kev.state & ModMask)
+				{
+					/*
+					 * VK_OEM_XXX doesn't cause WM_*_CHAR message
+					 * in WM_SYSKEYDOWN.
+					 */
+					kev.ch = oem_key_to_char( event->wparam) ;
+				}
+				else
+				{
+					kev.ch = 0 ;
+				}
+
+				if( kev.ch == 0 && event->wparam != VK_F10 /* Menu */)
+				{
+					break ;
+				}
 			}
 			else
 			{
-				kev.ch = 0 ;
-
-				if( ( kev.state = get_key_state()) & ControlMask)
+				/*
+				 * - See x_xic_get_str() in win32/x_xic.c.
+				 * - Following keys don't cause WM_*_CHAR message.
+				 */
+				if( ( kev.state & ControlMask) &&
+				    ( event->wparam == VK_OEM_2 ||	/* Ctl+/ */
+				      event->wparam == VK_OEM_3 ||	/* Ctl+@ */
+				      event->wparam == VK_OEM_7 ||	/* Ctl+^ */
+				      event->wparam == VK_OEM_102))	/* Ctl+_ */
 				{
-					/*
-					 * - See x_xic_get_str() in win32/x_xic.c.
-					 * - Following VK_* keys don't cause WM_*_CHAR message.
-					 */
-
-					/* Control+/ (106-JP Keyboard) */
-					if( event->wparam == VK_OEM_2)
-					{
-						kev.ch = '/' ;
-					}
-					/* Control+^ (106-JP Keyboard) */
-					else if( event->wparam == VK_OEM_7)
-					{
-						kev.ch = '^' ;
-					}
-					/* Control+_ (106-JP Keyboard) */
-					else if( event->wparam == VK_OEM_102)
-					{
-						kev.ch = '_' ;
-					}
+					kev.ch = oem_key_to_char( event->wparam) ;
+				}
+				else
+				{
+					kev.ch = 0 ;
 				}
 			}
 
 			(*win->key_pressed)( win , &kev) ;
+
+			/* Don't process VK_F10 as a menu key by DefWindowProc(). */
+			if( event->wparam == VK_F10)
+			{
+				return  1 ;
+			}
 		}
 
 		/* Continue default processing. */
@@ -1913,6 +2031,7 @@ x_window_receive_event(
 		if( win->parent == NULL)
 	#endif
 		{
+			urgent_bell( win , 0) ;
 			notify_focus_in_to_children( win) ;
 		}
 
@@ -1934,6 +2053,7 @@ x_window_receive_event(
 		break ;
 
 	case WM_MOUSEWHEEL:
+	case WM_MOUSEHWHEEL:
 	case WM_LBUTTONDOWN:
 	case WM_MBUTTONDOWN:
 	case WM_RBUTTONDOWN:
@@ -1982,7 +2102,8 @@ x_window_receive_event(
 				bev.state |= ControlMask ;
 			}
 
-			if( event->msg == WM_MOUSEWHEEL)
+			if( event->msg == WM_MOUSEWHEEL ||
+			    event->msg == WM_MOUSEHWHEEL)
 			{
 				/* Note that WM_MOUSEWHEEL event is reported to top window. */
 
@@ -1998,11 +2119,29 @@ x_window_receive_event(
 
 				if( ((SHORT)HIWORD(event->wparam)) > 0)
 				{
-					bev.button = 4 ;
+					if( event->msg == WM_MOUSEHWHEEL)
+					{
+						/* Right */
+						bev.button = Button7 ;
+					}
+					else
+					{
+						/* Up */
+						bev.button = Button4 ;
+					}
 				}
 				else
 				{
-					bev.button = 5 ;
+					if( event->msg == WM_MOUSEHWHEEL)
+					{
+						/* Left */
+						bev.button = Button6 ;
+					}
+					else
+					{
+						/* Down */
+						bev.button = Button5 ;
+					}
 				}
 			}
 			else
@@ -2012,17 +2151,17 @@ x_window_receive_event(
 
 				if( event->msg == WM_LBUTTONDOWN || event->msg == WM_LBUTTONUP)
 				{
-					bev.button = 1 ;
+					bev.button = Button1 ;
 				}
 				else if( event->msg == WM_MBUTTONDOWN ||
 					event->msg == WM_MBUTTONUP)
 				{
-					bev.button = 2 ;
+					bev.button = Button2 ;
 				}
 				else /* if( event->msg == WM_RBUTTONDOWN ||
 					event->msg == WM_RBUTTONUP) */
 				{
-					bev.button = 3 ;
+					bev.button = Button3 ;
 				}
 			}
 
@@ -2483,12 +2622,13 @@ int
 x_window_copy_area(
 	x_window_t *  win ,
 	Pixmap  src ,
-	int  src_x ,
-	int  src_y ,
+	PixmapMask  mask ,
+	int  src_x ,	/* >= 0 */
+	int  src_y ,	/* >= 0 */
 	u_int  width ,
 	u_int  height ,
-	int  dst_x ,
-	int  dst_y
+	int  dst_x ,	/* >= 0 */
+	int  dst_y	/* >= 0 */
 	)
 {
 	if( dst_x >= win->width || dst_y >= win->height)
@@ -2506,9 +2646,17 @@ x_window_copy_area(
 		height = win->height - dst_y ;
 	}
 
-	BitBlt( win->gc->gc , win->margin + dst_x , win->margin + dst_y ,
-		width , height ,
-		src , src_x , src_y , SRCCOPY) ;
+	if( mask)
+	{
+		MaskBlt( win->gc->gc , win->margin + dst_x , win->margin + dst_y ,
+			width , height , src , src_x , src_y , mask , src_x , src_y ,
+			MAKEROP4(SRCCOPY,PATCOPY)) ;
+	}
+	else
+	{
+		BitBlt( win->gc->gc , win->margin + dst_x , win->margin + dst_y ,
+			width , height , src , src_x , src_y , SRCCOPY) ;
+	}
 
 	return  1 ;
 }
@@ -2591,7 +2739,7 @@ x_window_draw_string(
 
 		if( ( dbl_str = alloca( len * 2)))
 		{
-			int  count ;
+			u_int  count ;
 
 			for( count = 0 ; count < len ; count++)
 			{
@@ -2667,7 +2815,7 @@ x_window_draw_image_string(
 
 		if( ( dbl_str = alloca( len * 2)))
 		{
-			int  count ;
+			u_int  count ;
 
 			for( count = 0 ; count < len ; count++)
 			{
@@ -2736,28 +2884,6 @@ x_window_draw_rect_frame(
 	return  1 ;
 }
 
-int
-x_window_draw_line(
-	x_window_t *  win,
-	int  x1,
-	int  y1,
-	int  x2,
-	int  y2
-	)
-{
-	if( win->gc->gc == None)
-	{
-		return  0 ;
-	}
-
-	x_release_pen( x_gc_set_pen( win->gc, x_acquire_pen( win->fg_color.pixel))) ;
-
-	MoveToEx( win->gc->gc, x1, y1, NULL) ;
-	LineTo( win->gc->gc, x2, y2) ;
-
-	return  1 ;
-}
-
 int
 x_set_use_clipboard_selection(
 	int  use_it
@@ -2898,7 +3024,7 @@ x_window_send_selection(
 {
 	HGLOBAL  hmem ;
 	u_char *  g_data ;
-	int  count ;
+	size_t  count ;
 
 	if( sel_data == NULL || sel_len == 0)
 	{
@@ -2944,19 +3070,14 @@ x_set_window_name(
 	{
 		name = root->app_name ;
 
-	#ifdef  UTF16_IME_CHAR
-		SetWindowTextW( root->my_window , name) ;
+	#ifndef  UTF16_IME_CHAR
+		SetWindowTextA( root->my_window , name) ;
 
 		return  1 ;
 	#endif
 	}
 
-	/*
-	 * XXX
-	 * Convert name to current locale encoding or UTF16.
-	 * Use SetWindowTextW if UTF16_IME_CHAR is defined.
-	 */
-	SetWindowTextA( root->my_window , name) ;
+	SetWindowTextW( root->my_window , name) ;
 
 	return  1 ;
 }
@@ -3019,14 +3140,6 @@ x_set_click_interval(
 	return  1 ;
 }
 
-XModifierKeymap *
-x_window_get_modifier_mapping(
-	x_window_t *  win
-	)
-{
-	return  x_display_get_modifier_mapping( win->disp) ;
-}
-
 u_int
 x_window_get_mod_ignore_mask(
 	x_window_t *  win ,
@@ -3046,12 +3159,24 @@ x_window_get_mod_meta_mask(
 }
 
 int
-x_window_bell(
-	x_window_t *  win ,
-	int  visual
+x_set_use_urgent_bell(
+	int  use
 	)
 {
-	if( visual)
+	use_urgent_bell = use ;
+
+	return  1 ;
+}
+
+int
+x_window_bell(
+	x_window_t *  win ,
+	x_bel_mode_t  mode
+	)
+{
+	urgent_bell( win , 1) ;
+
+	if( mode & BEL_VISUAL)
 	{
 		int  count ;
 
@@ -3070,7 +3195,8 @@ x_window_bell(
 		ReleaseDC( win->my_window, win->gc->gc) ;
 		x_set_gc( win->gc, None) ;
 	}
-	else
+
+	if( mode & BEL_SOUND)
 	{
 		Beep( 800 , 200) ;
 	}
@@ -3108,7 +3234,7 @@ x_window_dump_children(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 
 	kik_msg_printf( "%p(%li) => " , win , win->my_window) ;
 	for( count = 0 ; count < win->num_of_children ; count ++)
