@@ -9,7 +9,6 @@
 #include  <ml_color.h>
 
 #include  "x_color.h"
-#include  "x_color_config.h"
 #include  "x_color_cache.h"
 
 
@@ -24,8 +23,7 @@ typedef struct  x_color_manager
 	{
 		x_color_t  xcolor ;
 		char *  name ;
-		u_int8_t  is_loaded ;
-	
+
 	} sys_colors[6] ;
 
 	u_int8_t  alpha ;
@@ -35,7 +33,7 @@ typedef struct  x_color_manager
 
 
 x_color_manager_t *  x_color_manager_new( x_display_t *  disp ,
-	x_color_config_t *  color_config , char *  fg_color , char *  bg_color ,
+	char *  fg_color , char *  bg_color ,
 	char *  cursor_fg_color , char *  cursor_bg_color ,
 	char *  bd_color , char *  ul_color) ;
 
@@ -83,7 +81,7 @@ int  x_color_manager_adjust_bd_color( x_color_manager_t *  color_man) ;
 
 int  x_color_manager_adjust_ul_color( x_color_manager_t *  color_man) ;
 
-int  x_color_manager_unload( x_color_manager_t *  color_man) ;
+int  x_color_manager_reload( x_color_manager_t *  color_man) ;
 
 int  x_color_manager_change_alpha( x_color_manager_t *  color_man , u_int8_t  alpha) ;
 
