@@ -20,11 +20,13 @@
 #define IDD_USER	16
 #define IDD_PASS	17
 #define IDD_ENCODING	18
+#define IDD_EXEC_CMD	19
 #endif
 
 
-int  x_connect_dialog( char **  info , char **  pass , char *  display_name ,
-		Window  parent_window , char **  server_list , char *  default_server) ;
+int  x_connect_dialog( char **  info , char **  pass , char **  exec_cmd ,
+		char *  display_name , Window  parent_window ,
+		char **  server_list , char *  default_server) ;
 
 
 #endif
